import os
import sys
from functools import wraps

from dataclasses import dataclass
from fastapi import status
from sqlalchemy import func, or_
from sqlalchemy.orm import Session

from openjiuwen.core.common.logging import logger
from openjiuwen_studio.core.database import milliseconds
from openjiuwen_studio.core.manager.repositories import JiuwenBaseRepository
from openjiuwen_studio.core.manager.repositories.jiuwen_base_repository import escape_like, get_db_jw
from openjiuwen_studio.models import knowledge_base as kb_models
from openjiuwen_studio.models import knowledge_base_document as kb_doc_models
from openjiuwen_studio.schemas.common import ResponseModel
from openjiuwen_studio.schemas.knowledge_base import KnowledgeBaseGet
from openjiuwen_studio.core.config import settings


@dataclass
class KBDetails:
    space_id: str  # 空间ID
    kb_id: str | None = None  # 知识库ID
    index_manager_type: str | None = None  # milvus or chroma


@dataclass
class KBDocument:
    kb: KBDetails  # Knowledge Base details
    doc_id: str | None = None  # 文档ID
    doc_status: str = None  # 新状态
    process_info: dict | None = None  # 处理信息（可选）
    index_name: str | None = None  #
    chunk_count: int | None = None  #


class KnowledgeBaseRepository:
    def __init__(self) -> None:
        pass

    def with_exception_handling(func_):
        @wraps(func_)
        def wrapper(self, *args, **kwargs):
            try:
                return func_(self, *args, **kwargs)
            except Exception as e:
                exc_type, exc_value, exc_traceback = sys.exc_info()
                fname = os.path.split(exc_traceback.tb_frame.f_code.co_filename)[1]
                logger.error("Error: knowledge base db data preprocessing error")
                logger.debug(f"Exception details: {type(e).__name__}", exc_info=True)
                logger.error(f"{e}: {func_.__name__}({args=}, {kwargs=}) {exc_type} {fname}:{exc_traceback.tb_lineno}")
                return ResponseModel(
                    code=status.HTTP_400_BAD_REQUEST,
                    message=f"Error: knowledge base db data preprocessing error: {type(e).__name__}",
                )

        return wrapper

    """
    description: 创建知识库
    param {dict} kb_data  待创建的知识库数据
    return {*}
    """

    @with_exception_handling
    def knowledge_base_create(
        self, kb_data: dict, db_session: Session | None = None
    ) -> ResponseModel[None]:
        with get_db_jw(db_session) as db:
            kb_db = JiuwenBaseRepository(db, kb_models.KnowledgeBaseDB)
            if not kb_data:
                logger.debug(f"No knowledge base data to register: \ndata: {kb_data}")
                return ResponseModel(
                    code=status.HTTP_400_BAD_REQUEST, message="No knowledge base data to register"
                )

            find_id = {
                "kb_id": kb_data["kb_id"],
            }
            timestamp = milliseconds()
            if "create_time" not in kb_data or not kb_data["create_time"]:
                kb_data["create_time"] = timestamp
            if "update_time" not in kb_data or not kb_data["update_time"]:
                kb_data["update_time"] = timestamp

            return kb_db.register_dl_in_sql(find_id=find_id, dl=kb_data)

    """
    description: 从数据库获取知识库
    param {KnowledgeBaseGet} kb_get  知识库查询条件
    return {*}
    """

    @with_exception_handling
    def knowledge_base_get(
        self, kb_get: KnowledgeBaseGet, db_session: Session | None = None
    ) -> ResponseModel[dict | None]:
        with get_db_jw(db_session) as db:
            kb_db = JiuwenBaseRepository(db, kb_models.KnowledgeBaseDB)
            find_id = {
                "space_id": kb_get.space_id,
                "kb_id": kb_get.kb_id,
                "index_manager_type": kb_get.index_manager_type,
            }
            if kb_get.index_manager_type:
                find_id["index_manager_type"] = kb_get.index_manager_type
            return kb_db.get_dl_in_sql(find_id=find_id, return_first_item=True)

    """
    description: 删除知识库
    param {KnowledgeBaseGet} kb_get  知识库查询条件
    return {*}
    """

    @with_exception_handling
    def knowledge_base_delete(
        self, kb_get: KnowledgeBaseGet, db_session: Session | None = None
    ) -> ResponseModel[None]:
        with get_db_jw(db_session) as db:
            kb_db = JiuwenBaseRepository(db, kb_models.KnowledgeBaseDB)
            find_id = {
                "space_id": kb_get.space_id,
                "kb_id": kb_get.kb_id,
            }
            return kb_db.unregister_dl_in_sql(find_id=find_id)

    """
    description: 检查知识库名称是否已存在
    param {str} space_id  空间ID
    param {str} name  知识库名称
    param {str} exclude_kb_id  排除的知识库ID（用于更新时排除当前知识库）
    return {ResponseModel[bool]}  True表示名称已存在，False表示不存在
    """

    @with_exception_handling
    def knowledge_base_check_name_exists(
        self,
        space_id: str,
        name: str,
        exclude_kb_id: str | None = None,
        db_session: Session | None = None,
    ) -> ResponseModel[bool]:
        """检查知识库名称是否已存在（区分大小写）"""
        with get_db_jw(db_session) as db:
            # SQLite 默认区分大小写，MySQL 需要用 BINARY
            if settings.db_type.lower() == "sqlite":
                query = db.query(kb_models.KnowledgeBaseDB).filter(
                    kb_models.KnowledgeBaseDB.space_id == space_id,
                    kb_models.KnowledgeBaseDB.name == name,
                )
            else:
                query = db.query(kb_models.KnowledgeBaseDB).filter(
                    kb_models.KnowledgeBaseDB.space_id == space_id,
                    func.binary(kb_models.KnowledgeBaseDB.name) == func.binary(name),
                )
            if exclude_kb_id:
                query = query.filter(kb_models.KnowledgeBaseDB.kb_id != exclude_kb_id)

            exists = db.query(query.exists()).scalar()
            return ResponseModel(code=status.HTTP_200_OK, message="Success", data=exists)

    """
    description: 更新知识库
    param {str} space_id  空间ID
    param {str} kb_id  知识库ID
    param {str} name  新的名字
    param {str} description  新的描述
    return {*}
    """

    @with_exception_handling
    def knowledge_base_update(
        self,
        kb: KBDetails,
        name: str,
        description: str | None,
        db_session: Session | None = None,
    ) -> ResponseModel[None]:
        with get_db_jw(db_session) as db:
            kb_db = JiuwenBaseRepository(db, kb_models.KnowledgeBaseDB)
            find_id = {
                "space_id": kb.space_id,
                "kb_id": kb.kb_id,
            }

            if kb.index_manager_type:
                find_id["index_manager_type"] = kb.index_manager_type

            # 由于 update_dl_in_sql 使用 exclude_invalid=True 会过滤 None 和空字符串，
            # 如果 description 是 None 或空字符串，我们需要直接使用 SQLAlchemy 的 update 语句
            if description is None or description == "":
                from sqlalchemy import update as sql_update

                stmt = (
                    sql_update(kb_models.KnowledgeBaseDB)
                    .values(
                        name=name,
                        description=None if description is None else "",
                        update_time=milliseconds(),
                    )
                    .where(
                        kb_models.KnowledgeBaseDB.space_id == kb.space_id,
                        kb_models.KnowledgeBaseDB.kb_id == kb.kb_id,
                        kb_models.KnowledgeBaseDB.index_manager_type == kb.index_manager_type,
                    )
                )
                db.execute(stmt)
                db.commit()
                return ResponseModel(
                    code=status.HTTP_200_OK, message="Knowledge base updated successfully"
                )
            else:
                # 如果 description 有值，使用正常的更新流程
                update_data = {
                    "name": name,
                    "description": description,
                    "update_time": milliseconds(),
                }
                return kb_db.update_dl_in_sql(find_id=find_id, update_dl=update_data)

    """
    description: 创建知识库文档
    param {dict} doc_data  待创建的文档数据
    return {*}
    """

    @with_exception_handling
    def document_create(
        self, doc_data: dict, db_session: Session | None = None
    ) -> ResponseModel[None]:
        with get_db_jw(db_session) as db:
            doc_db = JiuwenBaseRepository(db, kb_doc_models.KnowledgeBaseDocumentDB)
            if not doc_data:
                logger.debug(f"No document data to register: \ndata: {doc_data}")
                return ResponseModel(
                    code=status.HTTP_400_BAD_REQUEST, message="No document data to register"
                )

            find_id = {
                "doc_id": doc_data["doc_id"],
            }
            timestamp = milliseconds()
            if "create_time" not in doc_data or not doc_data["create_time"]:
                doc_data["create_time"] = timestamp
            if "update_time" not in doc_data or not doc_data["update_time"]:
                doc_data["update_time"] = timestamp

            return doc_db.register_dl_in_sql(find_id=find_id, dl=doc_data)

    """
    description: 从数据库获取知识库文档
    param {str} space_id  空间ID
    param {str} kb_id  知识库ID
    param {str} doc_id  文档ID
    return {*}
    """

    @with_exception_handling
    def document_get(
        self,
        kbdoc: KBDocument,
        db_session: Session | None = None,
    ) -> ResponseModel[dict | None]:
        with get_db_jw(db_session) as db:
            doc_db = JiuwenBaseRepository(db, kb_doc_models.KnowledgeBaseDocumentDB)
            find_id = {
                "space_id": kbdoc.kb.space_id,
                "kb_id": kbdoc.kb.kb_id,
                "doc_id": kbdoc.doc_id,
            }
            if kbdoc.kb.index_manager_type:
                find_id["index_manager_type"] = kbdoc.kb.index_manager_type
            return doc_db.get_dl_in_sql(find_id=find_id, return_first_item=True)

    """
    description: 删除知识库文档
    param {str} space_id  空间ID
    param {str} kb_id  知识库ID
    param {str} doc_id  文档ID
    return {*}
    """

    @with_exception_handling
    def document_delete(
        self, kbdoc: KBDocument, db_session: Session | None = None
    ) -> ResponseModel[None]:
        with get_db_jw(db_session) as db:
            doc_db = JiuwenBaseRepository(db, kb_doc_models.KnowledgeBaseDocumentDB)
            find_id = {
                "space_id": kbdoc.kb.space_id,
                "kb_id": kbdoc.kb.kb_id,
                "doc_id": kbdoc.doc_id,
            }
            if kbdoc.kb.index_manager_type:
                find_id["index_manager_type"] = kbdoc.kb.index_manager_type
            return doc_db.unregister_dl_in_sql(find_id=find_id)

    @with_exception_handling
    def document_update_status(
        self, kbdoc: KBDocument, db_session: Session | None = None
    ) -> ResponseModel[None]:
        with get_db_jw(db_session) as db:
            doc_db = JiuwenBaseRepository(db, kb_doc_models.KnowledgeBaseDocumentDB)
            find_id = {
                "space_id": kbdoc.kb.space_id,
                "kb_id": kbdoc.kb.kb_id,
                "doc_id": kbdoc.doc_id,
            }
            if kbdoc.kb.index_manager_type:
                find_id["index_manager_type"] = kbdoc.kb.index_manager_type

            update_data = {
                "status": kbdoc.doc_status,
                "update_time": milliseconds(),
            }

            if kbdoc.process_info is not None:
                update_data["process_info"] = kbdoc.process_info

            # 如果提供了索引信息，一起更新
            if kbdoc.index_name is not None:
                update_data["index_id"] = None
                update_data["index_name"] = kbdoc.index_name
                update_data["indexed_time"] = milliseconds()

            if kbdoc.chunk_count is not None:
                update_data["chunk_count"] = kbdoc.chunk_count

            return doc_db.update_dl_in_sql(find_id=find_id, update_dl=update_data)

    """
    description: 更新文档信息（当前只支持更新文档名称）
    param {str} space_id  空间ID
    param {str} kb_id  知识库ID
    param {str} doc_id  文档ID
    param {str} name  新的文档名称
    return {*}
    """

    @with_exception_handling
    def document_update(
        self, kbdoc: KBDocument, name: str, db_session: Session | None = None
    ) -> ResponseModel[None]:
        with get_db_jw(db_session) as db:
            doc_db = JiuwenBaseRepository(db, kb_doc_models.KnowledgeBaseDocumentDB)
            find_id = {
                "space_id": kbdoc.kb.space_id,
                "kb_id": kbdoc.kb.kb_id,
                "doc_id": kbdoc.doc_id,
            }
            if kbdoc.kb.index_manager_type:
                find_id["index_manager_type"] = kbdoc.kb.index_manager_type
            update_data = {
                "name": name,
                "update_time": milliseconds(),
            }
            return doc_db.update_dl_in_sql(find_id=find_id, update_dl=update_data)

    """
    description: 查询知识库（查询词出现在名称或描述中，支持分页）
    param {str} space_id  空间ID
    param {str} query  查询词（查询词完整出现在知识库名称或描述中，大小写不敏感）
    param {int} page  页码，从1开始
    param {int} page_size  每页大小
    return {*}
    """

    @with_exception_handling
    def knowledge_base_search(
        self,
        space_id: str,
        query: str,
        page: int = 1,
        page_size: int = 10,
        index_manager_type: str | None = None,
        db_session: Session | None = None,
    ) -> ResponseModel[dict]:
        with get_db_jw(db_session) as db:
            kb_db = JiuwenBaseRepository(db, kb_models.KnowledgeBaseDB)

            # 构建查询条件：查询词完整出现在名称或描述中（大小写不敏感）
            # 使用 func.lower() + ilike 实现大小写不敏感匹配，escape_like 防止 LIKE 通配符注入
            query_lower = query.lower()
            escaped_query = escape_like(query_lower)
            search_conditions = or_(
                func.lower(kb_models.KnowledgeBaseDB.name).ilike(f"%{escaped_query}%", escape="\\"),
                func.lower(kb_models.KnowledgeBaseDB.description).ilike(f"%{escaped_query}%", escape="\\"),
            )

            # 构建基础查询条件
            find_id: dict = {
                "space_id": space_id,
            }
            if index_manager_type:
                find_id["index_manager_type"] = index_manager_type

            # 验证分页参数
            page = max(1, page)
            page_size = max(1, min(page_size, 100))  # 限制最大100

            # 计算 offset 和 limit
            offset = (page - 1) * page_size
            return_range = [offset, page_size]

            # 先获取总数（不分页）
            count_result = kb_db.get_dl_in_sql(
                find_id=find_id,
                other_sqlalchemy_limitations=[search_conditions],
                return_first_item=False,
            )

            # 如果返回 404 且消息是 "Data not found."，视为空结果
            if (
                count_result.code == status.HTTP_404_NOT_FOUND
                and count_result.message == "Data not found."
            ):
                total = 0
            elif count_result.code != status.HTTP_200_OK:
                return count_result
            else:
                total = len(count_result.data) if count_result.data else 0

            # 获取分页数据
            search_result = kb_db.get_dl_in_sql(
                find_id=find_id,
                other_sqlalchemy_limitations=[search_conditions],
                return_range=return_range,
                return_first_item=False,
            )

            # 如果返回 404 且消息是 "Data not found."，视为空结果
            if (
                search_result.code == status.HTTP_404_NOT_FOUND
                and search_result.message == "Data not found."
            ):
                knowledge_bases = []
            elif search_result.code != status.HTTP_200_OK:
                return search_result
            else:
                knowledge_bases = search_result.data or []

            # 计算总页数
            total_pages = max(1, (total + page_size - 1) // page_size) if total > 0 else 1

            # 返回分页结果
            return ResponseModel(
                code=status.HTTP_200_OK,
                message="Search knowledge bases successfully",
                data={
                    "knowledge_bases": knowledge_bases,
                    "total": total,
                    "page": page,
                    "page_size": page_size,
                    "total_pages": total_pages,
                },
            )

    """
    description: 获取知识库列表（支持分页）
    param {str} space_id  空间ID
    param {int} page  页码，从1开始
    param {int} size  每页大小
    return {*}
    """

    @with_exception_handling
    def knowledge_base_list(
        self,
        kb: KBDetails,
        page: int = 1,
        size: int = 10,
        db_session: Session | None = None,
    ) -> ResponseModel[dict]:
        with get_db_jw(db_session) as db:
            kb_db = JiuwenBaseRepository(db, kb_models.KnowledgeBaseDB)

            # 构建查询条件
            find_id = {
                "space_id": kb.space_id,
            }
            if kb.index_manager_type:
                find_id["index_manager_type"] = kb.index_manager_type
            # 计算分页参数
            offset = (page - 1) * size

            # 查询总数
            count_result = kb_db.get_dl_in_sql(find_id=find_id, return_first_item=False)
            if count_result.code == status.HTTP_404_NOT_FOUND:
                # 空数据是正常情况，返回空列表
                return ResponseModel(
                    code=status.HTTP_200_OK,
                    message="Get knowledge base list success",
                    data={"items": [], "total": 0},
                )
            elif count_result.code != status.HTTP_200_OK:
                return ResponseModel(
                    code=count_result.code,
                    message=count_result.message,
                    data={"items": [], "total": 0},
                )

            total = len(count_result.data) if count_result.data else 0

            # 查询分页数据
            query = (
                db.query(kb_models.KnowledgeBaseDB)
                .filter(kb_models.KnowledgeBaseDB.space_id == kb.space_id)
                .filter(kb_models.KnowledgeBaseDB.index_manager_type == kb.index_manager_type)
                .order_by(kb_models.KnowledgeBaseDB.create_time.desc())
                .offset(offset)
                .limit(size)
            )

            kb_list = query.all()

            # 转换为字典列表
            items = []
            for kb in kb_list:
                items.append(
                    {
                        "kb_id": kb.kb_id,
                        "space_id": kb.space_id,
                        "name": kb.name,
                        "description": kb.description,
                        "embedding_model_config_id": kb.embedding_model_config_id,
                        "config": kb.config,
                        "create_time": kb.create_time,
                        "update_time": kb.update_time,
                    }
                )

            return ResponseModel(
                code=status.HTTP_200_OK,
                message="Get knowledge base list success",
                data={"items": items, "total": total},
            )

    """
    description: 获取知识库文档列表（支持分页）
    param {str} space_id  空间ID
    param {str} kb_id  知识库ID
    param {int} page  页码，从1开始
    param {int} size  每页大小
    return {*}
    """

    @with_exception_handling
    def document_list(
        self,
        kbdoc: KBDocument,
        page: int = 1,
        size: int = 10,
        db_session: Session | None = None,
    ) -> ResponseModel[dict]:
        with get_db_jw(db_session) as db:
            doc_db = JiuwenBaseRepository(db, kb_doc_models.KnowledgeBaseDocumentDB)

            # 构建查询条件
            find_id = {
                "space_id": kbdoc.kb.space_id,
                "kb_id": kbdoc.kb.kb_id,
            }

            # 如果传入 index_manager_type，则将其加入筛选条件（用于 count 和列表查询）
            if kbdoc.kb.index_manager_type:
                find_id["index_manager_type"] = kbdoc.kb.index_manager_type

            # 计算分页参数
            offset = (page - 1) * size

            # 查询总数
            count_result = doc_db.get_dl_in_sql(find_id=find_id, return_first_item=False)
            if count_result.code == status.HTTP_404_NOT_FOUND:
                # 空数据是正常情况，返回空列表
                return ResponseModel(
                    code=status.HTTP_200_OK,
                    message="Get document list success",
                    data={"items": [], "total": 0, "page": page, "size": size},
                )
            elif count_result.code != status.HTTP_200_OK:
                return ResponseModel(
                    code=count_result.code,
                    message=count_result.message,
                    data={"items": [], "total": 0, "page": page, "size": size},
                )

            total = len(count_result.data) if count_result.data else 0

            query = db.query(kb_doc_models.KnowledgeBaseDocumentDB).filter(
                kb_doc_models.KnowledgeBaseDocumentDB.space_id == kbdoc.kb.space_id,
                kb_doc_models.KnowledgeBaseDocumentDB.kb_id == kbdoc.kb.kb_id,
            )
            if kbdoc.kb.index_manager_type:
                query = query.filter(
                    kb_doc_models.KnowledgeBaseDocumentDB.index_manager_type
                    == kbdoc.kb.index_manager_type
                )
            query = (
                query.order_by(kb_doc_models.KnowledgeBaseDocumentDB.create_time.desc())
                .offset(offset)
                .limit(size)
            )

            doc_list = query.all()

            # 转换为字典列表
            items = []
            for doc in doc_list:
                items.append(
                    {
                        "doc_id": doc.doc_id,
                        "name": doc.name,
                        "status": doc.status,
                        "process_info": doc.process_info if doc.process_info else {},
                        "create_time": doc.create_time,
                        "update_time": doc.update_time,
                    }
                )

            return ResponseModel(
                code=status.HTTP_200_OK,
                message="Get document list success",
                data={"items": items, "total": total, "page": page, "size": size},
            )

    """
    description: 检查知识库是否有图增强构建的文档
    param {str} space_id  空间ID
    param {str} kb_id  知识库ID
    return {bool} 是否有图增强文档
    """

    @with_exception_handling
    def has_graph_enhancement_documents(
        self, space_id: str, kb_id: str, db_session: Session | None = None
    ) -> bool:
        """检查知识库中是否有图增强构建的文档。"""
        with get_db_jw(db_session) as db:
            # 查询该知识库下所有已索引的文档
            docs = (
                db.query(kb_doc_models.KnowledgeBaseDocumentDB)
                .filter(
                    kb_doc_models.KnowledgeBaseDocumentDB.space_id == space_id,
                    kb_doc_models.KnowledgeBaseDocumentDB.kb_id == kb_id,
                    kb_doc_models.KnowledgeBaseDocumentDB.status == "indexed",
                )
                .all()
            )

            # 检查是否有文档使用了图增强
            for doc in docs:
                if doc.process_info and isinstance(doc.process_info, dict):
                    indexing_strategy = doc.process_info.get("indexing_strategy")
                    if isinstance(indexing_strategy, dict):
                        enable_graph_enhancement = indexing_strategy.get(
                            "enable_graph_enhancement", False
                        )
                        if enable_graph_enhancement:
                            return True

            return False


# 创建全局实例
knowledge_base_repository = KnowledgeBaseRepository()



from functools import wraps

from fastapi import status
from sqlalchemy import func, or_
from sqlalchemy.orm import Session

from openjiuwen_studio.core.database import milliseconds
from openjiuwen.core.common.logging import logger
from openjiuwen_studio.core.manager.repositories import JiuwenBaseRepository
from openjiuwen_studio.core.manager.repositories.jiuwen_base_repository import escape_like, get_db_jw
from openjiuwen_studio.models import memory_base as mb_models
from openjiuwen_studio.schemas.common import ResponseModel
from openjiuwen_studio.schemas.memory_base import MemoryBaseGet
from openjiuwen_studio.core.config import settings


class MemoryBaseRepository:
    def __init__(self) -> None:
        pass

    @staticmethod
    def with_exception_handling(func_):
        @wraps(func_)
        def wrapper(self, *args, **kwargs):
            try:
                return func_(self, *args, **kwargs)
            except Exception as e:
                logger.error("Error: memory base db data preprocessing error")
                logger.debug(f"Exception details: {type(e).__name__}", exc_info=True)
                return ResponseModel(
                    code=status.HTTP_400_BAD_REQUEST,
                    message=f"Error: memory base db data preprocessing error: {type(e).__name__}"
                )

        return wrapper

    '''
    description: 创建记忆库
    param {dict} mb_data  待创建的记忆库数据
    return {*}
    '''

    @with_exception_handling
    def memory_base_create(self, mb_data: dict, db_session: Session | None = None) -> ResponseModel[None]:
        with get_db_jw(db_session) as db:
            mb_db = JiuwenBaseRepository(db, mb_models.MemoryBaseDB)
            if not mb_data:
                logger.debug(f"No memory base data to register: \ndata: {mb_data}")
                return ResponseModel(code=status.HTTP_400_BAD_REQUEST, message="No memory base data to register")

            find_id = {
                "mdb_id": mb_data["mdb_id"],
            }
            timestamp = milliseconds()
            if "create_time" not in mb_data or not mb_data["create_time"]:
                mb_data["create_time"] = timestamp
            if "update_time" not in mb_data or not mb_data["update_time"]:
                mb_data["update_time"] = timestamp

            return mb_db.register_dl_in_sql(find_id=find_id, dl=mb_data)

    '''
    description: 从数据库获取记忆库
    param {MemoryBaseGet} mb_get  记忆库查询条件
    return {*}
    '''

    @with_exception_handling
    def memory_base_get(
        self, mb_get: MemoryBaseGet, db_session: Session | None = None
    ) -> ResponseModel[dict | None]:
        with get_db_jw(db_session) as db:
            mb_db = JiuwenBaseRepository(db, mb_models.MemoryBaseDB)
            find_id = {
                "space_id": mb_get.space_id,
                "mdb_id": mb_get.mdb_id,
            }
            return mb_db.get_dl_in_sql(find_id=find_id, return_first_item=True)

    '''
    description: 根据数据库id从数据库获取记忆库
    param  mdb_id  记忆库查询条件
    return {*}
    '''

    @with_exception_handling
    def memory_base_get_by_id(
        self, mdb_id: str, db_session: Session | None = None
    ) -> ResponseModel[dict | None]:
        with get_db_jw(db_session) as db:
            mb_db = JiuwenBaseRepository(db, mb_models.MemoryBaseDB)
            find_id = {
                "mdb_id": mdb_id,
            }
            return mb_db.get_dl_in_sql(find_id=find_id, return_first_item=True)

    '''
    description: 删除记忆库
    param {MemoryBaseGet} mb_get  记忆库查询条件
    return {*}
    '''

    @with_exception_handling
    def memory_base_delete(self, mb_get: MemoryBaseGet, db_session: Session | None = None) -> ResponseModel[None]:
        with get_db_jw(db_session) as db:
            mb_db = JiuwenBaseRepository(db, mb_models.MemoryBaseDB)
            find_id = {
                "space_id": mb_get.space_id,
                "mdb_id": mb_get.mdb_id,
            }
            return mb_db.unregister_dl_in_sql(find_id=find_id)

    '''
    description: 检查记忆库名称是否已存在
    param {str} space_id  空间ID
    param {str} name  记忆库名称
    param {str} exclude_mdb_id  排除的记忆库ID（用于更新时排除当前记忆库）
    return {ResponseModel[bool]}  True表示名称已存在，False表示不存在
    '''

    @with_exception_handling
    def memory_base_check_name_exists(
        self,
        space_id: str,
        name: str,
        exclude_mdb_id: str | None = None,
        db_session: Session | None = None,
    ) -> ResponseModel[bool]:
        with get_db_jw(db_session) as db:
            query = db.query(mb_models.MemoryBaseDB).filter(
                mb_models.MemoryBaseDB.space_id == space_id
            )

            # 根据数据库类型选择不同的比较方式
            if settings.db_type.lower() == "sqlite":
                # SQLite 默认不区分大小写，所以需要特殊处理
                query = query.filter(func.lower(mb_models.MemoryBaseDB.name) == func.lower(name))
            else:
                # MySQL 等数据库默认区分大小写
                query = query.filter(mb_models.MemoryBaseDB.name == name)

            # 如果提供了排除的记忆库ID，则排除该记录
            if exclude_mdb_id:
                query = query.filter(mb_models.MemoryBaseDB.mdb_id != exclude_mdb_id)

            # 检查是否存在匹配的记录
            exists = db.query(query.exists()).scalar()

            return ResponseModel(code=status.HTTP_200_OK, message="Success", data=exists)

    '''
    description: 更新记忆库
    param {dict} mb_data  待更新的记忆库数据
    param {MemoryBaseGet} mb_get  记忆库查询条件
    return {*}
    '''

    @with_exception_handling
    def memory_base_update(self, mb_data: dict, mb_get: MemoryBaseGet, db_session: Session | None = None) -> \
        ResponseModel[None]:
        with get_db_jw(db_session) as db:
            mb_db = JiuwenBaseRepository(db, mb_models.MemoryBaseDB)
            if not mb_data:
                logger.debug(f"No memory base data to update: \ndata: {mb_data}")
                return ResponseModel(code=status.HTTP_400_BAD_REQUEST, message="No memory base data to update")

            # 更新时间戳
            mb_data["update_time"] = milliseconds()

            find_id = {
                "space_id": mb_get.space_id,
                "mdb_id": mb_get.mdb_id,
            }

            return mb_db.update_dl_in_sql(find_id=find_id, update_dl=mb_data)

    '''
    description: 获取记忆库列表（支持分页）
    param {str} space_id  空间ID
    param {int} page  页码，从1开始
    param {int} size  每页大小
    return {*}
    '''

    @with_exception_handling
    def memory_base_list(
        self,
        space_id: str,
        page: int = 1,
        size: int = 10,
        db_session: Session | None = None
    ) -> ResponseModel[dict]:
        with get_db_jw(db_session) as db:
            mb_db = JiuwenBaseRepository(db, mb_models.MemoryBaseDB)

            # 构建查询条件
            find_id = {
                "space_id": space_id,
            }

            # 计算分页参数
            offset = (page - 1) * size

            # 查询总数
            count_result = mb_db.get_dl_in_sql(find_id=find_id, return_first_item=False)
            if count_result.code == status.HTTP_404_NOT_FOUND:
                # 空数据是正常情况，返回空列表
                return ResponseModel(
                    code=status.HTTP_200_OK,
                    message="Get memory base list success",
                    data={"items": [], "total": 0}
                )
            elif count_result.code != status.HTTP_200_OK:
                return ResponseModel(
                    code=count_result.code,
                    message=count_result.message,
                    data={"items": [], "total": 0}
                )

            total = len(count_result.data) if count_result.data else 0

            # 查询分页数据
            query = db.query(mb_models.MemoryBaseDB).filter(
                mb_models.MemoryBaseDB.space_id == space_id
            ).order_by(
                mb_models.MemoryBaseDB.update_time.desc()
            ).offset(offset).limit(size)

            mb_list = query.all()

            # 转换为字典列表
            items = []
            for mb in mb_list:
                items.append({
                    "mdb_id": mb.mdb_id,
                    "space_id": mb.space_id,
                    "name": mb.name,
                    "description": mb.description,
                    "embedding_model_config_id": mb.embedding_model_config_id,
                    "llm_model_config_id": mb.llm_model_config_id,
                    "create_time": mb.create_time,
                    "update_time": mb.update_time,
                })

            return ResponseModel(
                code=status.HTTP_200_OK,
                message="Get memory base list success",
                data={
                    "items": items,
                    "total": total
                }
            )

    '''
    description: 搜索记忆库（支持按名称和描述模糊查询，分页）
    param {str} space_id  空间ID
    param {str} query  查询词（查询词完整出现在记忆库名称或描述中，大小写不敏感）
    param {int} page  页码，从1开始
    param {int} page_size  每页大小，最大100
    return {*}
    '''

    @with_exception_handling
    def memory_base_search(
        self,
        space_id: str,
        query: str,
        page: int = 1,
        page_size: int = 10,
        db_session: Session | None = None
    ) -> ResponseModel[dict]:
        with get_db_jw(db_session) as db:
            mb_db = JiuwenBaseRepository(db, mb_models.MemoryBaseDB)

            # 构建查询条件：查询词完整出现在名称或描述中（大小写不敏感）
            # 使用 func.lower() + ilike 实现大小写不敏感匹配，escape_like 防止 LIKE 通配符注入
            query_lower = query.lower()
            escaped_query = escape_like(query_lower)
            search_conditions = or_(
                func.lower(mb_models.MemoryBaseDB.name).ilike(f"%{escaped_query}%", escape="\\"),
                func.lower(mb_models.MemoryBaseDB.description).ilike(f"%{escaped_query}%", escape="\\")
            )

            # 构建基础查询条件
            find_id = {
                "space_id": space_id,
            }

            # 验证分页参数
            page = max(1, page)
            page_size = max(1, min(page_size, 100))  # 限制最大100

            # 计算 offset 和 limit
            offset = (page - 1) * page_size
            return_range = [offset, page_size]

            # 先获取总数（不分页）
            count_result = mb_db.get_dl_in_sql(
                find_id=find_id,
                other_sqlalchemy_limitations=[search_conditions],
                return_first_item=False
            )

            # 如果返回 404 且消息是 "Data not found."，视为空结果
            if count_result.code == status.HTTP_404_NOT_FOUND and count_result.message == "Data not found.":
                total = 0
            elif count_result.code != status.HTTP_200_OK:
                return count_result
            else:
                total = len(count_result.data) if count_result.data else 0

            # 获取分页数据
            search_result = mb_db.get_dl_in_sql(
                find_id=find_id,
                other_sqlalchemy_limitations=[search_conditions],
                return_range=return_range,
                return_first_item=False,
                order_cols_desc=["update_time"]  # 添加排序参数
            )

            # 如果返回 404 且消息是 "Data not found."，视为空结果
            if search_result.code == status.HTTP_404_NOT_FOUND and search_result.message == "Data not found.":
                memory_bases = []
            elif search_result.code != status.HTTP_200_OK:
                return search_result
            else:
                memory_bases = search_result.data or []

            # 计算总页数
            total_pages = max(1, (total + page_size - 1) // page_size) if total > 0 else 1

            # 返回分页结果
            return ResponseModel(
                code=status.HTTP_200_OK,
                message="Search memory bases successfully",
                data={
                    "memory_bases": memory_bases,
                    "total": total,
                    "page": page,
                    "page_size": page_size,
                    "total_pages": total_pages
                }
            )

# 创建全局实例
memory_base_repository = MemoryBaseRepository()


from functools import wraps
from typing import Any, Callable, Dict, List, Optional

from fastapi import status
from sqlalchemy import and_, delete, func, or_, select
from sqlalchemy.orm import Session

from openjiuwen.core.common.logging import logger
from openjiuwen_studio.core.database import milliseconds
from openjiuwen_studio.core.manager.repositories.jiuwen_base_repository import (
    JiuwenBaseRepository, get_db_jw, get_val_from_dict)
from openjiuwen_studio.models.tag import TagDB, workflow_tag_association
from openjiuwen_studio.schemas.common import ResponseModel


class WorkflowTagRepository:
    """Workflow-Tag association data access layer."""

    def __init__(self) -> None:
        pass

    def with_exception_handling(func_) -> Callable:
        @wraps(func_)
        def wrapper(self, *args, **kwargs):
            try:
                return func_(self, *args, **kwargs)
            except Exception as e:
                logger.error(f"Error: workflow-tag db data processing failed, {str(e)}")
                return ResponseModel(
                    code=status.HTTP_400_BAD_REQUEST,
                    message=f"Error: workflow-tag db data processing failed, {str(e)}"
                ).model_dump(exclude_none=True)

        return wrapper

    @with_exception_handling
    def associate_tag(self, workflow_data: dict) -> dict:
        """Associate a tag with a workflow.

        Args:
            workflow_data: Association data containing workflow_id, workflow_version, tag_id, space_id

        Returns:
            ResponseModel indicating success/failure
        """
        with get_db_jw() as db:
            if not workflow_data:
                logger.debug(f"No tag data to register: \ndata: {workflow_data}")
                return ResponseModel(
                    code=status.HTTP_400_BAD_REQUEST,
                    message="No tag data to register"
                ).model_dump(exclude_none=True)

            # Check required fields
            required_fields = ["workflow_id", "workflow_version", "tag_id", "space_id"]
            for field in required_fields:
                if field not in workflow_data or not workflow_data[field]:
                    return ResponseModel(
                        code=status.HTTP_400_BAD_REQUEST,
                        message=f"Missing required field: {field}"
                    ).model_dump(exclude_none=True)

            # Check if association already exists (including version)
            find_id = {
                "workflow_id": workflow_data["workflow_id"],
                "workflow_version": workflow_data["workflow_version"],
                "tag_id": workflow_data["tag_id"],
                "space_id": workflow_data["space_id"]
            }

            existing_assoc = WorkflowTagRepository._get_association(db, find_id)
            if existing_assoc.code == status.HTTP_200_OK:
                return ResponseModel(
                    code=status.HTTP_400_BAD_REQUEST,
                    message=f"Association already exists"
                ).model_dump(exclude_none=True)

            # Set timestamps
            timestamp = milliseconds()
            if "create_time" not in workflow_data:
                workflow_data["create_time"] = timestamp
            if "workflow_version" not in workflow_data:
                workflow_data["workflow_version"] = ""

            # Create association
            try:
                db.execute(workflow_tag_association.insert().values(workflow_data))
                db.commit()
                return ResponseModel(
                    code=status.HTTP_200_OK,
                    message="Tag association created successfully"
                ).model_dump(exclude_none=True)
            except Exception as e:
                db.rollback()
                logger.error(f"Tag association creation failed: {str(e)}")
                return ResponseModel(
                    code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                    message=f"Tag association creation failed: {str(e)}"
                ).model_dump(exclude_none=True)

    @with_exception_handling
    def get_workflow_tags(self, query_body: dict) -> dict:
        """Get all tags associated with a workflow.

        Args:
            query_body: Query containing workflow_id and space_id

        Returns:
            ResponseModel with list of associated tags
        """
        with get_db_jw() as db:
            workflow_id = get_val_from_dict(query_body, ["workflow_id"])
            space_id = get_val_from_dict(query_body, ["space_id"])
            workflow_version = get_val_from_dict(query_body, ["workflow_version"])

            if not workflow_id or not space_id:
                return ResponseModel(
                    code=status.HTTP_400_BAD_REQUEST,
                    message="workflow_id and space_id are required"
                ).model_dump(exclude_none=True)

            try:
                # Get tag associations with version support
                query = select(workflow_tag_association.c.tag_id).where(
                    workflow_tag_association.c.workflow_id == workflow_id,
                    workflow_tag_association.c.space_id == space_id
                )

                # Add version filter if provided
                if workflow_version:
                    query = query.where(workflow_tag_association.c.workflow_version == workflow_version)

                associations = db.execute(query).fetchall()

                tag_ids = [assoc[0] for assoc in associations]

                if not tag_ids:
                    return ResponseModel(
                        code=status.HTTP_200_OK,
                        message="No tags found for workflow",
                        data=[]
                    ).model_dump(exclude_none=True)

                # Get tag details
                tags_result = db.execute(
                    select(TagDB).where(TagDB.primary_id.in_(tag_ids))
                ).fetchall()

                tag_list = []
                logger.info(f"Found {len(tags_result)} tags for tag_ids: {tag_ids}")

                for tag_row in tags_result:
                    logger.info(f"Processing tag: {tag_row}, type: {type(tag_row)}")
                    # Extract TagDB object from Row
                    tag = tag_row[0]  # First element in the row is the TagDB object
                    tag_dict = {
                        "primary_id": tag.primary_id,
                        "space_id": tag.space_id,
                        "tag_name": tag.tag_name,
                        "tag_color": tag.tag_color,
                        "is_active": tag.is_active,
                        "usage_count": tag.usage_count,
                        "create_time": tag.create_time,
                        "update_time": tag.update_time,
                        "create_user": tag.create_user,
                        "update_user": tag.update_user
                    }
                    tag_list.append(tag_dict)

                return ResponseModel(
                    code=status.HTTP_200_OK,
                    message="Tags found for workflow",
                    data=tag_list
                ).model_dump(exclude_none=True)

            except Exception as e:
                logger.error(f"Error getting workflow tags: {str(e)}")
                return ResponseModel(
                    code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                    message=f"Error getting workflow tags: {str(e)}"
                ).model_dump(exclude_none=True)

    @with_exception_handling
    def remove_workflow_tags(self, query_body: dict) -> dict:
        """Remove all tag associations for a workflow.

        Args:
            query_body: Query containing workflow_id and space_id

        Returns:
            ResponseModel indicating success/failure
        """
        with get_db_jw() as db:
            try:
                workflow_id = get_val_from_dict(query_body, ["workflow_id"])
                space_id = get_val_from_dict(query_body, ["space_id"])
                workflow_version = get_val_from_dict(query_body, ["workflow_version"])

                if not workflow_id or not space_id:
                    return ResponseModel(
                        code=status.HTTP_400_BAD_REQUEST,
                        message="workflow_id and space_id are required"
                    ).model_dump(exclude_none=True)

                # Build delete query with version support
                delete_query = delete(workflow_tag_association).where(
                    workflow_tag_association.c.workflow_id == workflow_id,
                    workflow_tag_association.c.space_id == space_id,
                    workflow_tag_association.c.workflow_version == workflow_version
                )

                # # Add version filter if provided
                # if workflow_version:
                #     delete_query = delete_query.where(workflow_tag_association.c.workflow_version == workflow_version)

                db.execute(delete_query)
                db.commit()
                return ResponseModel(
                    code=status.HTTP_200_OK,
                    message="Workflow tags removed successfully"
                ).model_dump(exclude_none=True)
            except Exception as e:
                db.rollback()
                logger.error(f"Error removing workflow tags: {str(e)}")
                return ResponseModel(
                    code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                    message=f"Error removing workflow tags: {str(e)}"
                ).model_dump(exclude_none=True)

    @with_exception_handling
    def remove_tag_association(self, query_body: dict) -> dict:
        """Remove specific tag associations from a workflow.

        Args:
            query_body: Query containing workflow_id, space_id, and tag_ids

        Returns:
            ResponseModel indicating success/failure
        """
        with get_db_jw() as db:
            workflow_id = get_val_from_dict(query_body, ["workflow_id"])
            space_id = get_val_from_dict(query_body, ["space_id"])
            tag_ids = get_val_from_dict(query_body, ["tag_ids"])

            if not workflow_id or not space_id or not tag_ids:
                return ResponseModel(
                    code=status.HTTP_400_BAD_REQUEST,
                    message="workflow_id, space_id, and tag_ids are required"
                ).model_dump(exclude_none=True)

            if not isinstance(tag_ids, list):
                tag_ids = [tag_ids]

            try:
                db.execute(
                    delete(workflow_tag_association).where(
                        workflow_tag_association.c.workflow_id == workflow_id,
                        workflow_tag_association.c.space_id == space_id,
                        workflow_tag_association.c.tag_id.in_(tag_ids)
                    )
                )
                db.commit()
                return ResponseModel(
                    code=status.HTTP_200_OK,
                    message="Tag association removed successfully"
                ).model_dump(exclude_none=True)
            except Exception as e:
                db.rollback()
                logger.error("Error removing tag association")
                return ResponseModel(
                    code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                    message=f"Error removing tag association: {str(e)}"
                ).model_dump(exclude_none=True)

    @with_exception_handling
    def find_workflows_by_tags(self, query_body: dict) -> dict:
        """Get workflow IDs that have any of the specified tags.

        Args:
            query_body: Query containing space_id and tag_ids

        Returns:
            ResponseModel with list of workflow IDs that have the specified tags
        """
        with get_db_jw() as db:
            space_id = get_val_from_dict(query_body, ["space_id"])
            tag_ids = get_val_from_dict(query_body, ["tag_ids"])

            if not space_id or not tag_ids:
                return ResponseModel(
                    code=status.HTTP_400_BAD_REQUEST,
                    message="space_id and tag_ids are required"
                ).model_dump(exclude_none=True)

            if not isinstance(tag_ids, list):
                tag_ids = [tag_ids]

            try:
                # Get workflow IDs that have any of the specified tags
                # We need to get distinct workflow_ids, ignoring workflow_version to get all versions
                query = select(workflow_tag_association.c.workflow_id).where(
                    workflow_tag_association.c.space_id == space_id,
                    workflow_tag_association.c.tag_id.in_(tag_ids)
                ).distinct()

                logger.info(f"Executing tag search query for space_id: {space_id}, tag_ids: {tag_ids}")
                associations = db.execute(query).fetchall()
                workflow_ids = [assoc[0] for assoc in associations]
                logger.info(f"Tag search returned {len(workflow_ids)} workflow IDs")

                return ResponseModel(
                    code=status.HTTP_200_OK,
                    message="Workflows found for tags",
                    data=workflow_ids
                ).model_dump(exclude_none=True)

            except Exception as e:
                logger.error(
                    f"Error finding workflows by tags for space_id {space_id}, tag_ids {tag_ids}: {str(e)}")
                return ResponseModel(
                    code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                    message=f"Error finding workflows by tags: {str(e)}"
                ).model_dump(exclude_none=True)

    @staticmethod
    def _get_association(db: Session, find_id: dict) -> ResponseModel:
        """Helper method to get tag association.

        Args:
            db: Database session
            find_id: Query parameters containing workflow_id, workflow_version, tag_id, space_id

        Returns:
            ResponseModel with association data if found
        """
        try:
            workflow_id = find_id.get("workflow_id")
            workflow_version = find_id.get("workflow_version")
            tag_id = find_id.get("tag_id")
            space_id = find_id.get("space_id")

            if not all([workflow_id, workflow_version, tag_id, space_id]):
                return ResponseModel(
                    code=status.HTTP_400_BAD_REQUEST,
                    message="workflow_id, workflow_version, tag_id, and space_id are required"
                )

            association = db.execute(
                select(workflow_tag_association).where(
                    workflow_tag_association.c.workflow_id == workflow_id,
                    workflow_tag_association.c.workflow_version == workflow_version,
                    workflow_tag_association.c.tag_id == tag_id,
                    workflow_tag_association.c.space_id == space_id
                )
            ).fetchone()

            if association:
                # Extract data from SQLAlchemy Row object by column name
                association_data = {
                    "workflow_id": association.workflow_id,
                    "workflow_version": association.workflow_version,
                    "tag_id": association.tag_id,
                    "space_id": association.space_id,
                    "create_time": association.create_time
                }
                return ResponseModel(
                    code=status.HTTP_200_OK,
                    message="Association found",
                    data=association_data
                )
            else:
                return ResponseModel(
                    code=status.HTTP_404_NOT_FOUND,
                    message="Association not found"
                )

        except Exception as e:
            logger.error(f"Error getting tag association: {str(e)}")
            return ResponseModel(
                code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                message=f"Error getting association: {str(e)}"
            )


# Create singleton instance
workflow_tag_repository = WorkflowTagRepository()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (c) Huawei Technologies Co., Ltd. 2025-2025. All rights reserved.
from typing import Any, Dict

from openjiuwen_studio.core.common import dsl
from openjiuwen_studio.core.manager.convertor.components.common import outputs_convert, input_params_convert
from openjiuwen_studio.core.common.dsl import ComponentType
from openjiuwen_studio.schemas.node import Node, Outputs, BaseValue


def text_editor_convert(node: Node) -> dsl.Component:
    try:
        data = node.data
        inputs = data.inputs
        if inputs is None:
            raise ValueError("text_editor_convert inputs is none")
        input_parameters = inputs.input_parameters
        if input_parameters is None:
            raise ValueError("text_editor_convert input_parameters is empty")

        converted_inputs = input_params_convert(input_parameters)

        data_outputs = data.outputs
        if data_outputs is None:
            raise ValueError("text_editor_convert outputs is none")
        converted_outputs = outputs_convert(data_outputs)

        text_editor_param = inputs.text_editor_param
        if text_editor_param is None:
            raise ValueError("text_editor_param is empty")
        delimiters = (text_editor_param.delimiters or []) + (text_editor_param.custom_delimiters or [])
        text_configs = dsl.TextEditorConfig(
            edit_type=text_editor_param.edit_type,
            delimiters=delimiters,
            concatenate_format=(
                text_editor_param.concatenate_format.content if text_editor_param.concatenate_format else None
            )
        )

        # 构建转换后的LLM组件配置
        text_editor_component = dsl.Component(
            id=getattr(node, 'id', ""),
            type=ComponentType.COMPONENT_TYPE_TEXT_EDITOR,
            type_version="1.0.0",
            inputs=converted_inputs,
            outputs=converted_outputs,
            configs=text_configs.model_dump(),
            name=data.title
        )
        return text_editor_component
    except Exception as e:
        raise RuntimeError(f"Failed to convert text_editor: {str(e)}") from e



from sqlalchemy import BigInteger, Boolean, Integer, JSON, String, UniqueConstraint, Index
from sqlalchemy.orm import Mapped, mapped_column
from openjiuwen_studio.models.db_fun_base import Base, DBFunBase
from openjiuwen_studio.ops.config import settings


# ==================== user 表 ====================
class UserDB(Base, DBFunBase):
    __tablename__ = "user"
    __table_args__ = (
        UniqueConstraint("email", name="uniq_email"),
        UniqueConstraint("user_id", name="uniq_user_id"),
        UniqueConstraint("user_unique_name", name="uniq_user_unique_name"),
    )

    # 主键 
    if settings.DB_TYPE.lower() == "sqlite":
        id: Mapped[int] = mapped_column(
            Integer, primary_key=True, autoincrement=True, comment="Primary Key ID"
        )
    else:
        id: Mapped[int] = mapped_column(
            BigInteger, primary_key=True, autoincrement=True, comment="Primary Key ID"
        )

    # 必填字段
    user_id_str: Mapped[str] = mapped_column(String(100), comment="USER ID", nullable=False, name="user_id")
    email: Mapped[str] = mapped_column(String(128), default="", nullable=False)
    user_unique_name: Mapped[str] = mapped_column(String(128), default="", nullable=False)
    username: Mapped[str] = mapped_column(String(128), default="", nullable=False, name="user_name")
    password: Mapped[str] = mapped_column(String(128), default="", nullable=False)
    session_key: Mapped[str] = mapped_column(String(512), default="", nullable=False)
    refresh_token: Mapped[str] = mapped_column(String(512), server_default="", nullable=True)
    role_type: Mapped[int] = mapped_column(Integer, default=0, nullable=False)
    user_verified: Mapped[int] = mapped_column(Boolean, default=False, nullable=False)
    is_active: Mapped[int] = mapped_column(Boolean, default=False, nullable=False)

    # 可选字段
    description: Mapped[str | None] = mapped_column(String(512), default=None, nullable=True)
    avatar_url: Mapped[str | None] = mapped_column(String(512), default=None, nullable=True, name="icon_uri")
    locale: Mapped[str | None] = mapped_column(String(128), default=None, nullable=True)
    first_name: Mapped[str | None] = mapped_column(String(128), default=None, nullable=True)
    last_name: Mapped[str | None] = mapped_column(String(128), default=None, nullable=True)
    phone_number: Mapped[str | None] = mapped_column(String(128), default=None, nullable=True)
    company: Mapped[str | None] = mapped_column(String(128), default=None, nullable=True)
    occupation: Mapped[str | None] = mapped_column(String(512), default=None, nullable=True)
    skills: Mapped[list | dict | None] = mapped_column(JSON, default=None, nullable=True)
    _rest_: Mapped[list | dict | None] = mapped_column(JSON, default=None, nullable=True)

    # 时间戳
    user_create_time: Mapped[int | None] = mapped_column(BigInteger, default=None, nullable=True, name="create_time")
    user_update_time: Mapped[int | None] = mapped_column(BigInteger, default=None, nullable=True, name="update_time")
    user_deleted_time: Mapped[int | None] = mapped_column(BigInteger, default=None, nullable=True, name="delete_time")

    def __repr__(self) -> str:
        return (
            f"<{self.__tablename__}("
            f"email='{self.email}', "
            f"user_id={self.user_id_str}, "
            f"user_unique_name={self.user_unique_name}, "
            f"user_verified='{self.user_verified}', "
            f"is_active={self.is_active})>"
        )


class SpaceDB(Base, DBFunBase):
    __tablename__ = "space"
    __table_args__ = (
        UniqueConstraint("space_id", name="uniq_space_id"),
        Index("idx_creator_id", "creator_id"),
        Index("idx_owner_id", "owner_id"),
    )

    if settings.DB_TYPE.lower() == "sqlite":
        id: Mapped[int] = mapped_column(
            Integer, primary_key=True, autoincrement=True, comment="Primary Key ID"
        )
    else:
        id: Mapped[int] = mapped_column(
            BigInteger, primary_key=True, autoincrement=True, comment="Primary Key ID"
        )

    space_id: Mapped[str] = mapped_column(String(100), nullable=False)
    user_id_str: Mapped[str] = mapped_column(String(100), nullable=False, name="owner_id")
    spacename: Mapped[str | None] = mapped_column(String(200), default=None, nullable=True, name="space_name")
    icon_url: Mapped[str | None] = mapped_column(String(200), default=None, nullable=True, name="icon_uri")
    creator_id_str: Mapped[str | None] = mapped_column(String(100), default=None, nullable=True, name="creator_id")
    description: Mapped[str | None] = mapped_column(String(2000), default=None, nullable=True)
    _rest_: Mapped[list | dict | None] = mapped_column(JSON, default=None, nullable=True)

    # 时间戳
    space_create_time: Mapped[int | None] = mapped_column(BigInteger, default=None, nullable=True, name="create_time")
    space_update_time: Mapped[int | None] = mapped_column(BigInteger, default=None, nullable=True, name="update_time")
    space_deleted_time: Mapped[int | None] = mapped_column(BigInteger, default=None, nullable=True, name="delete_time")

    def __repr__(self) -> str:
        return (
            f"<{self.__tablename__}("
            f"id={self.id}, "
            f"space_id={self.space_id}, "
            f"owner_id={self.user_id_str})>"
        )


class SpaceUserDB(Base, DBFunBase):
    __tablename__ = "space_user"
    __table_args__ = (
        UniqueConstraint("space_id", "user_id", name="uniq_space_user"),
        Index("idx_user_id", "user_id"),
    )

    if settings.DB_TYPE.lower() == "sqlite":
        id: Mapped[int] = mapped_column(
            Integer, primary_key=True, autoincrement=True, comment="Primary Key ID"
        )
    else:
        id: Mapped[int] = mapped_column(
            BigInteger, primary_key=True, autoincrement=True, comment="Primary Key ID"
        )

    space_id: Mapped[str] = mapped_column(String(100), nullable=False, comment="Space ID")
    user_id_str: Mapped[str] = mapped_column(String(100), nullable=False, comment="User ID", name="user_id")
    # 时间戳
    user_create_time: Mapped[int | None] = mapped_column(BigInteger, default=None, nullable=True, name="created_time")
    user_update_time: Mapped[int | None] = mapped_column(BigInteger, default=None, nullable=True, name="update_time")

    def __repr__(self) -> str:
        return (
            f"<{self.__tablename__}("
            f"id={self.id}, "
            f"user_id={self.user_id_str})>"
        )




from typing import Optional

from fastapi import APIRouter, Depends, HTTPException, Query, status
from sqlalchemy.orm import Session

from openjiuwen.core.common.logging import logger
from openjiuwen_studio.core.database import get_db
from openjiuwen_studio.core.manager.model_manager.managers.embedding_model_config_manager import \
    EmbeddingModelConfigManager
from openjiuwen_studio.core.manager.model_manager.managers.embedding_model_test_manager import \
    EmbeddingModelTester
from openjiuwen_studio.core.exceptions import (
    ModelConfigNotFoundError,
    ModelTestError,
    ValidationError
)
from openjiuwen_studio.routers.auth import get_current_user
from openjiuwen_studio.schemas.common import ResponseModel
from openjiuwen_studio.schemas.embedding_model_config import (
    EmbeddingModelConfigCreate, EmbeddingModelConfigList,
    EmbeddingModelConfigRequest, EmbeddingModelConfigResponse,
    EmbeddingModelConfigUpdate, EmbeddingModelConfigUpdateRequest,
    EmbeddingModelTestRequest, EmbeddingProtocol)

embedding_models_router = APIRouter(prefix="/embedding-models", tags=["embedding-models"])


def get_embedding_model_config_manager(db: Session = Depends(get_db)) -> EmbeddingModelConfigManager:
    return EmbeddingModelConfigManager(db)


def get_embedding_model_tester(db: Session = Depends(get_db)) -> EmbeddingModelTester:
    return EmbeddingModelTester(db)


def embedding_model_to_response(manager: EmbeddingModelConfigManager, model) -> EmbeddingModelConfigResponse:
    """转换为响应格式"""
    return manager._model_to_response(model)


@embedding_models_router.get("/{space_id}", response_model=ResponseModel[EmbeddingModelConfigList])
async def get_embedding_model_configs(
    space_id: str,
    page: int = Query(1, ge=1, description="Page number"),
    size: int = Query(10, ge=1, le=100, description="Page size"),
    protocol: Optional[EmbeddingProtocol] = Query(None, description="Filter by protocol"),
    is_active: Optional[bool] = Query(None, description="Filter by active status"),
    search: Optional[str] = Query(None, description="Search in model_name, model_id"),
    sort_by: Optional[str] = Query("updated_at", description="Sort by field"),
    sort_order: Optional[str] = Query("desc", description="Sort order (asc, desc)"),
    manager: EmbeddingModelConfigManager = Depends(get_embedding_model_config_manager),
    current_user: dict = Depends(get_current_user)
):
    """获取 Embedding 模型配置列表"""
    try:
        models, total = manager.get_paginated_configs(
            space_id=space_id,
            page=page,
            size=size,
            protocol=protocol,
            is_active=is_active,
            search=search,
            sort_by=sort_by,
            sort_order=sort_order
        )
        
        # 转换为响应格式
        model_responses = [manager._model_to_response(model) for model in models]
        
        result = EmbeddingModelConfigList(
            items=model_responses,
            total=total,
            page=page,
            size=size
        )
        
        return ResponseModel(
            code=200,
            message="Embedding model configurations retrieved successfully",
            data=result
        )
    except Exception as e:
        logger.error(f"Error retrieving embedding model configs: {str(e)}")
        raise HTTPException(status_code=500, detail="Internal server error") from e


@embedding_models_router.get("/", response_model=ResponseModel[EmbeddingModelConfigResponse])
async def get_embedding_model_config(
    config_id: int = Query(..., description="配置ID"),
    space_id: str = Query(..., description="空间ID"),
    manager: EmbeddingModelConfigManager = Depends(get_embedding_model_config_manager),
    current_user: dict = Depends(get_current_user)
):
    """获取单个 Embedding 模型配置"""
    try:
        model = manager.get_config_by_id(config_id, space_id)
        return ResponseModel(
            code=200,
            message="Embedding model configuration retrieved successfully",
            data=manager._model_to_response(model)
        )
    except Exception as e:
        if "not found" in str(e).lower():
            raise HTTPException(
                status_code=status.HTTP_404_NOT_FOUND,
                detail="Embedding model configuration not found"
            ) from e
        logger.error(f"Error retrieving embedding model config: {str(e)}")
        raise HTTPException(status_code=500, detail="Internal server error") from e


@embedding_models_router.post("/", response_model=ResponseModel[EmbeddingModelConfigResponse])
async def create_embedding_model_config(
    model_config: EmbeddingModelConfigCreate,
    manager: EmbeddingModelConfigManager = Depends(get_embedding_model_config_manager),
    current_user: dict = Depends(get_current_user)
):
    """创建 Embedding 模型配置"""
    try:
        db_model = manager.create_config(model_config)
        return ResponseModel(
            code=200,
            message="Embedding model configuration created successfully",
            data=manager._model_to_response(db_model)
        )
    except Exception as e:
        logger.error(f"Error creating embedding model config: {str(e)}")
        if "already exists" in str(e).lower():
            raise HTTPException(
                status_code=status.HTTP_400_BAD_REQUEST,
                detail="Embedding model configuration with this name already exists"
            ) from e
        raise HTTPException(status_code=500, detail="Internal server error") from e


@embedding_models_router.post("/update", response_model=ResponseModel[EmbeddingModelConfigResponse])
async def update_embedding_model_config(
    request: EmbeddingModelConfigUpdateRequest,
    manager: EmbeddingModelConfigManager = Depends(get_embedding_model_config_manager),
    current_user: dict = Depends(get_current_user)
):
    """更新 Embedding 模型配置"""
    try:
        # 创建更新对象（排除 config_id 和 space_id）
        update_data = request.model_dump(exclude={'config_id', 'space_id'}, exclude_unset=True)
        model_config_update = EmbeddingModelConfigUpdate(**update_data)
        
        # 更新配置
        model = manager.update_config(request.config_id, request.space_id, model_config_update)
        
        return ResponseModel(
            code=200,
            message="Embedding model configuration updated successfully",
            data=manager._model_to_response(model)
        )
    except Exception as e:
        if "not found" in str(e).lower():
            raise HTTPException(
                status_code=status.HTTP_404_NOT_FOUND,
                detail="Embedding model configuration not found"
            ) from e
        elif "already exists" in str(e).lower():
            raise HTTPException(
                status_code=status.HTTP_400_BAD_REQUEST,
                detail="Embedding model configuration with this model_name already exists"
            ) from e
        logger.error(f"Error updating embedding model config: {str(e)}")
        raise HTTPException(status_code=500, detail="Internal server error") from e


@embedding_models_router.delete("/", response_model=ResponseModel[dict])
async def delete_embedding_model_config(
    model_request: EmbeddingModelConfigRequest,
    manager: EmbeddingModelConfigManager = Depends(get_embedding_model_config_manager),
    current_user: dict = Depends(get_current_user)
):
    """删除 Embedding 模型配置"""
    try:
        success = manager.delete_config(model_request.config_id, model_request.space_id)
        return ResponseModel(
            code=200,
            message="Embedding model configuration deleted successfully",
            data={"deleted_id": model_request.config_id}
        )
    except Exception as e:
        if "not found" in str(e).lower():
            raise HTTPException(
                status_code=status.HTTP_404_NOT_FOUND,
                detail="Embedding model configuration not found"
            ) from e
        elif "in use" in str(e).lower() or "being used" in str(e).lower():
            raise HTTPException(
                status_code=status.HTTP_400_BAD_REQUEST,
                detail=str(e)
            ) from e
        logger.error(f"Error deleting embedding model config: {str(e)}")
        raise HTTPException(status_code=500, detail="Internal server error") from e


@embedding_models_router.post("/{config_id}/test", response_model=ResponseModel[dict])
async def test_embedding_model_config(
    config_id: int,
    test_request: EmbeddingModelTestRequest,
    tester: EmbeddingModelTester = Depends(get_embedding_model_tester),
    current_user: dict = Depends(get_current_user)
):
    """测试 Embedding 模型配置 - 直接返回 API 原始响应"""
    try:
        api_response = await tester.test_embedding_model(
            model_id=config_id,
            test_request=test_request,
            user_id=current_user["data"]["user_id_str"]
        )
        
        # 检查是否有错误
        if "error" in api_response:
            raise HTTPException(
                status_code=500,
                detail=api_response.get("error", "Embedding API call failed")
            )
        
        # 直接返回 API 的原始响应
        return ResponseModel(
            code=200,
            message="Embedding model test completed",
            data=api_response
        )
        
    except ModelConfigNotFoundError as e:
        # 模型配置不存在，透传错误信息
        raise HTTPException(status_code=404, detail=str(e)) from e
    except ModelTestError as e:
        # 模型测试失败，透传错误信息（包含详细的配置问题，如 API key、model name、URL 等）
        raise HTTPException(status_code=400, detail=str(e)) from e
    except ValidationError as e:
        # 数据验证失败，透传错误信息
        raise HTTPException(status_code=400, detail=str(e)) from e
    except ValueError as e:
        if "not found" in str(e).lower():
            raise HTTPException(status_code=404, detail=str(e)) from e
        else:
            raise HTTPException(status_code=400, detail=str(e)) from e
    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"Error testing embedding model configuration: {str(e)}", exc_info=True)
        # 透传异常信息，而不是通用的 "Internal server error"
        raise HTTPException(status_code=500, detail=str(e)) from e


@embedding_models_router.post("/toggle", response_model=ResponseModel[EmbeddingModelConfigResponse])
async def toggle_embedding_model_status(
    request: EmbeddingModelConfigRequest,
    manager: EmbeddingModelConfigManager = Depends(get_embedding_model_config_manager),
    current_user: dict = Depends(get_current_user)
):
    """切换 Embedding 模型配置的激活状态"""
    try:
        updated_model = manager.toggle_status(request.config_id, request.space_id)
        return ResponseModel(
            code=200,
            message=f"Embedding model {'activated' if updated_model.is_active else 'deactivated'} successfully",
            data=manager._model_to_response(updated_model)
        )
    except Exception as e:
        if "not found" in str(e).lower():
            raise HTTPException(
                status_code=status.HTTP_404_NOT_FOUND,
                detail="Embedding model configuration not found"
            ) from e
        logger.error(f"Error toggling embedding model status: {str(e)}")
        raise HTTPException(status_code=500, detail="Internal server error") from e




#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (c) Huawei Technologies Co., Ltd. 2025-2025. All rights reserved.
from __future__ import annotations
import sys
from pathlib import Path

sys.path.append(str(Path(__file__).resolve().parent.parent.parent))
from typing import Any

from sqlalchemy import (JSON, BigInteger, Boolean, ForeignKey, ForeignKeyConstraint,
                        Index, Integer, String, Text, UniqueConstraint)
from sqlalchemy.orm import (Mapped, declarative_mixin,
                            declared_attr, mapped_column, relationship)

from openjiuwen_studio.models.db_fun_base import Base, DBFunBase
from openjiuwen_studio.ops.config import settings


@declarative_mixin
class PluginDBMixin:
    if settings.DB_TYPE.lower() == "sqlite":
        primary_id: Mapped[int] = mapped_column(Integer, primary_key=True, autoincrement=True, name="id")
    else:
        primary_id: Mapped[int] = mapped_column(BigInteger, primary_key=True, autoincrement=True, name="id")

    name: Mapped[str | None] = mapped_column(String(255), nullable=True, name="plugin_name")
    desc: Mapped[str | None] = mapped_column(String(512), nullable=True, name="desc")
    desc_mk: Mapped[str | None] = mapped_column(Text, nullable=True, default=None)
    url: Mapped[str | None] = mapped_column(String(512), nullable=True, default=None)
    space_id: Mapped[str | None] = mapped_column(String(100), nullable=True)
    icon_uri: Mapped[str | None] = mapped_column(String(512), nullable=True, default=None)
    plugin_type: Mapped[int] = mapped_column(Integer, default=0, nullable=False)

    # 通用 JSON，MySQL/PostgreSQL/SQLite 均支持
    tools: Mapped[list[dict] | None] = mapped_column(JSON, default=None, nullable=True)
    inputs: Mapped[list[dict] | None] = mapped_column(JSON, default=None, nullable=True, name="inputs")
    _rest_: Mapped[dict | None] = mapped_column(JSON, default=None, nullable=True)

    create_time: Mapped[int | None] = mapped_column(BigInteger, nullable=True)
    update_time: Mapped[int | None] = mapped_column(BigInteger, nullable=True)


# ==================== tbl_plugin ====================
class PluginBaseDB(PluginDBMixin, Base, DBFunBase):
    __tablename__ = "plugin"
    __table_args__ = (
        UniqueConstraint(
            "plugin_id", "plugin_version",
            name="unique_plugin_id_version"
        ),
        Index("idx_space_id", "space_id"),
    )

    plugin_id: Mapped[str] = mapped_column(String(100), index=True, unique=True, nullable=False)
    plugin_version: Mapped[str | None] = mapped_column(String(100), default=DBFunBase.__version_none__)

    @declared_attr
    def plugin_publish_list(cls):
        return relationship(
            "PluginPublishDB",
            back_populates="plugin_draft",
            cascade="all, delete-orphan",
            passive_deletes=True,
        )

    @declared_attr
    def tool_list(cls):
        return relationship(
            "ToolBaseDB",
            back_populates="plugin_info",
            cascade="all, delete-orphan",
            passive_deletes=True,
        )

    def __repr__(self) -> str:
        return (
            f"<{self.__tablename__}("
            f"space_id='{self.space_id}', "
            f"plugin_id={self.plugin_id}, "
            f"plugin_version={self.plugin_version}, "
            f"plugin_name='{self.name})>"
        )


# ==================== tbl_plugin_publish ====================
class PluginPublishDB(PluginDBMixin, Base, DBFunBase):
    __tablename__ = "plugin_publish"
    __table_args__ = (
        UniqueConstraint(
            "plugin_id", "plugin_version",
            name="unique_plugin_id_version"
        ),
        ForeignKeyConstraint(
            ['plugin_id'],  # 子表的列
            ['plugin.plugin_id'],  # 父表的列
            ondelete="CASCADE"  # 数据库级联删除
        ),
        Index("idx_space_id", "space_id"),
    )

    plugin_id: Mapped[str] = mapped_column(String(100),
                                    ForeignKey("plugin.plugin_id", ondelete="CASCADE"),
                                    nullable=False,
                                    index=True,
                                    )
    plugin_version: Mapped[str] = mapped_column(String(100), nullable=False)
    version_desc: Mapped[str | None] = mapped_column(String(512))

    plugin_draft: Mapped[PluginBaseDB] = relationship(
        "PluginBaseDB",
        foreign_keys=plugin_id,
        back_populates="plugin_publish_list",
    )

    def __repr__(self) -> str:
        return (
            f"<{self.__tablename__}("
            f"space_id='{self.space_id}', "
            f"workflow_id={self.plugin_id}, "
            f"workflow_version={self.plugin_version}, "
            f"workflow_name='{self.name})>"
        )


PluginBaseDBPd = PluginBaseDB.sqlalchemy_to_pydantic(exclude={"primary_id", })
PluginPublishDBPd = PluginPublishDB.sqlalchemy_to_pydantic(exclude={"primary_id", })


class ToolBaseDB(Base, DBFunBase):
    __tablename__ = "tool"
    __table_args__ = (
        UniqueConstraint(
            "tool_id", "plugin_version",
            name="unique_tool_id_version"
        ),
        ForeignKeyConstraint(
            ['plugin_id', 'plugin_version'],  # 子表的列
            ['plugin.plugin_id', 'plugin.plugin_version'],  # 父表的列
            ondelete="CASCADE"  # 数据库级联删除
        ),
        Index("idx_space_id", "space_id"),
    )

    if settings.DB_TYPE.lower() == "sqlite":
        primary_id: Mapped[int] = mapped_column(Integer, primary_key=True, autoincrement=True, name="id")
    else:
        primary_id: Mapped[int] = mapped_column(BigInteger, primary_key=True, autoincrement=True, name="id")

    tool_id: Mapped[str] = mapped_column(String(100), nullable=False)
    name: Mapped[str | None] = mapped_column(String(255), nullable=True, name="tool_name")
    desc: Mapped[str | None] = mapped_column(String(512), nullable=True, name="desc")
    space_id: Mapped[str | None] = mapped_column(String(100), nullable=True)
    plugin_id: Mapped[str] = mapped_column(String(100), nullable=False)
    plugin_type: Mapped[int] = mapped_column(Integer, default=0, nullable=False)
    plugin_version: Mapped[str | None] = mapped_column(String(100), default=DBFunBase.__version_none__)

    input_parameters: Mapped[list[dict[str, Any]] | None] = mapped_column(JSON, default=None, nullable=True)
    output_parameters: Mapped[list[dict[str, Any]] | None] = mapped_column(JSON, default=None, nullable=True)
    available: Mapped[bool] = mapped_column(Boolean, default=False, nullable=False)
    _rest_: Mapped[dict | None] = mapped_column(JSON, default=None, nullable=True)

    create_time: Mapped[int | None] = mapped_column(BigInteger, nullable=True)
    update_time: Mapped[int | None] = mapped_column(BigInteger, nullable=True)
    plugin_info: Mapped[PluginBaseDB] = relationship(
        "PluginBaseDB",
        back_populates="tool_list",
        foreign_keys=[plugin_id, plugin_version]
    )

    def __repr__(self) -> str:
        return (
            f"<{self.__tablename__}("
            f"space_id='{self.space_id}', "
            f"tool_id={self.tool_id}, "
            f"plugin_version={self.plugin_version}, "
            f"tool_name='{self.name})>"
        )


#!/usr/bin/python3.10
# -*- coding: utf-8 -*-
# Copyright (c) Huawei Technologies Co., Ltd. 2025-2025. All rights reserved.
from fastapi import APIRouter, Depends, status, Path, Body, Query
from sqlalchemy.orm import Session

from openjiuwen_studio.ops.common.handle_exceptions_util import handle_exceptions
from openjiuwen_studio.ops.modules.prompt.application.service import PromptService
from openjiuwen_studio.ops.modules.prompt.domain import entities
from openjiuwen_studio.ops.modules.prompt.domain.entities import BatchGetPromptResponse, BatchGetPromptRequest
from openjiuwen_studio.ops.modules.prompt.infra.repositories import SQLPromptUserDraftRepository
from openjiuwen_studio.ops.modules.prompt.infra.repositories import SQLPromptRepository
from openjiuwen_studio.ops.modules.prompt.infra.database import get_db_ops, get_db_agent
from openjiuwen_studio.ops.modules.prompt.infra.repositories.agent_repo import SQLAgentRepository
from openjiuwen_studio.ops.modules.prompt.infra.repositories.prompt_repo import SQLPromptSubmitRepository
from openjiuwen_studio.core.manager.login_manager.space import check_user_space
from openjiuwen_studio.core.manager.login_manager.user import get_current_user

router = APIRouter(prefix="/api/v1/prompts", tags=["prompts"])


def get_prompt_service(db: Session = Depends(get_db_ops), agent_db: Session = Depends(get_db_agent)) -> PromptService:
    """ 依赖注入，获取 PromptService 实例 """
    prompt_repo = SQLPromptRepository(db)
    prompt_user_draft_repo = SQLPromptUserDraftRepository(db)
    prompt_commit_repo = SQLPromptSubmitRepository(db)
    agent_repo = SQLAgentRepository(agent_db)
    return PromptService(prompt_repo, prompt_user_draft_repo, prompt_commit_repo, agent_repo)


@router.post("/", response_model=entities.CreatePromptResponse, status_code=status.HTTP_201_CREATED)
@handle_exceptions(response_model=entities.CreatePromptResponse)
def create_prompt(
        new_prompt: entities.CreatePromptRequest,
        service: PromptService = Depends(get_prompt_service),
        current_user: dict = Depends(get_current_user)
):
    """ 创建新的 Prompt """
    _ = check_user_space(str(new_prompt.workspace_id), current_user)
    return service.create_prompt(new_prompt)


@router.post("/list", response_model=entities.ListPromptResponse)
@handle_exceptions(response_model=entities.ListPromptResponse)
def list_prompts(
        list_prompt: entities.ListPromptRequest,
        service: PromptService = Depends(get_prompt_service),
        current_user: dict = Depends(get_current_user)):
    """ 列出所有 Prompts """
    _ = check_user_space(str(list_prompt.workspace_id), current_user)
    return service.list_prompts(list_prompt)


@router.get("/{prompt_id}", response_model=entities.GetPromptResponse)
@handle_exceptions(response_model=entities.GetPromptResponse)
def get_prompt(
        prompt_id: int = Path(..., title="Prompt ID"),
        workspace_id: int = Query(..., title="Space ID"),
        with_draft: bool = Query(True, title="return with draft"),
        with_commit: bool = Query(True, title="return with commit"),
        commit_version: str = Query("-1", title="commit version"),
        with_default_config: bool = Query(True, title="return with default_config"),
        service: PromptService = Depends(get_prompt_service),
        current_user: dict = Depends(get_current_user)):
    """ 获取指定 Prompt 详情 """
    prompts = {"prompt_id": prompt_id, "with_draft": with_draft, "with_commit": with_commit,
               "with_default_config": with_default_config, "commit_version": commit_version}
    _ = check_user_space(str(workspace_id), current_user)
    return service.get_prompt(prompts)


@router.get("/{prompt_id}", response_model=entities.PromptWithVersions)
@handle_exceptions(response_model=entities.PromptWithVersions)
def get_prompt_with_version(
        prompt_id: int,
        workspace_id: int,
        service: PromptService = Depends(get_prompt_service),
        current_user: dict = Depends(get_current_user)):
    """ 获取指定 Prompt 及其版本历史 """
    prompts = {"prompt_id": prompt_id}
    _ = check_user_space(str(workspace_id), current_user)
    return service.get_prompt(prompts)


@router.put("/{prompt_id}", response_model=entities.UpdatePromptResponse)
@handle_exceptions(response_model=entities.UpdatePromptResponse)
def update_prompt(
        new_prompt: entities.UpdatePromptRequest,
        service: PromptService = Depends(get_prompt_service),
        current_user: dict = Depends(get_current_user)
):
    """ 更新指定 Prompt """
    _ = check_user_space(str(new_prompt.workspace_id), current_user)
    return service.update_prompt(new_prompt)


@router.delete("/{prompt_id}", response_model=entities.DeletePromptResponse)
@handle_exceptions(response_model=entities.DeletePromptResponse)
def delete_prompt(
        prompts: entities.DeletePromptRequest,
        service: PromptService = Depends(get_prompt_service),
        current_user: dict = Depends(get_current_user)):
    """ 删除指定 Prompt """
    _ = check_user_space(str(prompts.workspace_id), current_user)
    return service.delete_prompt(prompts)


@router.post("/{prompt_id}/clone", response_model=entities.ClonePromptResponse, status_code=status.HTTP_201_CREATED)
@handle_exceptions(response_model=entities.ClonePromptResponse)
def clone_prompt(
        prompt_id: str,
        new_prompt: entities.ClonePromptRequest,
        service: PromptService = Depends(get_prompt_service),
        current_user: dict = Depends(get_current_user)):
    """ 创建新的 Prompt """
    _ = check_user_space(str(new_prompt.workspace_id), current_user)
    new_prompt.user_id = current_user.get("data")["user_id_str"]
    return service.clone_prompt(prompt_id, new_prompt)


@router.post("/{prompt_id}/drafts/save", response_model=entities.DraftSaveResponse)
@handle_exceptions(response_model=entities.DraftSaveResponse)
def save_draft(
        prompt_id: int = Path(..., title="Prompt ID"),
        draft_input: entities.PromptDraftInput = Body(...),
        service: PromptService = Depends(get_prompt_service),
        current_user: dict = Depends(get_current_user)
):
    """
    保存Prompt草稿
    """
    _ = check_user_space(draft_input.prompt_draft.draft_info.space_id, current_user)
    draft_input.prompt_draft.draft_info.user_id = current_user.get("data")["user_id_str"]
    draft_info = service.save_draft(prompt_id, draft_input)
    return entities.DraftSaveResponse(code=0, msg="Success", draft_info=draft_info)


@router.get("/{prompt_id}/drafts", response_model=entities.PromptDraftInput)
@handle_exceptions(response_model=entities.PromptDraftInput)
def get_draft(
        prompt_id: int = Path(..., title="Prompt ID"),
        workspace_id: int = Query(..., title="Space ID"),
        service: PromptService = Depends(get_prompt_service),
        current_user: dict = Depends(get_current_user)
):
    """
    获取用户草稿
    """
    _ = check_user_space(str(workspace_id), current_user)
    user_id = current_user.get("data")["user_id_str"]
    draft = service.get_draft(prompt_id, user_id)
    return draft


@router.post("/{prompt_id}/drafts/commit", response_model=entities.CommitResponse)
@handle_exceptions(response_model=entities.CommitResponse)
def commit_draft(
        prompt_id: int = Path(..., title="Prompt ID"),
        workspace_id: int = Query(..., title="Space ID"),
        commit_request: entities.CommitRequest = Body(...),
        service: PromptService = Depends(get_prompt_service),
        current_user: dict = Depends(get_current_user)
):
    """
    提交草稿为正式版本
    """
    _ = check_user_space(str(workspace_id), current_user)
    user_id = current_user.get("data")["user_id_str"]
    service.commit_draft(
        prompt_id=prompt_id,
        user_id=user_id,
        commit_version=commit_request.commit_version,
        commit_description=commit_request.commit_description
    )

    return entities.CommitResponse(code=0, msg="success")


@router.get("/{prompt_id}/commits/list", response_model=entities.CommitListResponse)
@handle_exceptions(response_model=entities.CommitListResponse)
def list_commits(
        prompt_id: int = Path(..., title="Prompt ID"),
        workspace_id: int = Query(..., title="Space ID"),
        page_size: int = Query(10, title="Page Size"),
        service: PromptService = Depends(get_prompt_service),
        current_user: dict = Depends(get_current_user)
):
    """
    获取提交记录列表
    """
    _ = check_user_space(str(workspace_id), current_user)
    commit_infos = service.list_commits(prompt_id, page_size)

    return entities.CommitListResponse(
        code=0,
        msg="success",
        prompt_commit_infos=commit_infos
    )


@router.post("/{prompt_id}/drafts/revert_from_commit", response_model=entities.RevertFromCommitResponse)
@handle_exceptions(response_model=entities.RevertFromCommitResponse)
def revert_from_commit(
        prompt_id: int = Path(..., title="Prompt ID"),
        workspace_id: int = Query(..., title="Space ID"),
        revert_request: entities.RevertFromCommitRequest = Body(...),
        service: PromptService = Depends(get_prompt_service),
        current_user: dict = Depends(get_current_user)
):
    """
    从提交记录恢复草稿
    """
    _ = check_user_space(str(workspace_id), current_user)
    user_id = current_user.get("data")["user_id_str"]
    service.revert_from_commit(
        prompt_id=prompt_id,
        user_id=user_id,
        commit_version=revert_request.commit_version_reverting_from
    )

    return entities.RevertFromCommitResponse(code=0, msg="success")


@router.post("/batch-get", response_model=BatchGetPromptResponse)
@handle_exceptions(response_model=BatchGetPromptResponse)
def batch_get_prompts(
    request: BatchGetPromptRequest = Body(...),
    service: PromptService = Depends(get_prompt_service),
    current_user: dict = Depends(get_current_user)
):
    """
    批量获取prompt
    """
    _ = check_user_space(str(request.workspace_id), current_user)
    items = service.batch_get_prompts(request)
    return BatchGetPromptResponse(items=items, code=0, msg="success")



#!/usr/bin/python3.10
# -*- coding: utf-8 -*-
# Copyright (c) Huawei Technologies Co., Ltd. 2025-2025. All rights reserved.

from fastapi import APIRouter, Depends, HTTPException
from sqlalchemy.orm import Session

from openjiuwen_studio.core.manager.login_manager.user import get_current_user
from openjiuwen_studio.core.manager.login_manager.space import check_user_space
from openjiuwen_studio.ops.modules.llm.llm_config_service import LLMConfigService
from openjiuwen_studio.ops.config import ModelConfigManager
from openjiuwen_studio.ops.modules.llm.llm_manager import init_llm_manager
from openjiuwen_studio.ops.modules.llm.schema import ListModelRequest, GetModelRequest
from openjiuwen_studio.ops.modules.prompt.infra.database import get_db_agent
from openjiuwen_studio.ops.modules.prompt.infra.repositories.agent_repo import SQLAgentRepository

router = APIRouter(
    prefix="/api/v1/llm",
    tags=["model"],
)

router_initialized = False          # 模块级标记，确保只 init 一次


def get_llm_config_service(agent_db: Session = Depends(get_db_agent)) -> LLMConfigService:
    """
    依赖注入：创建 LLMConfigService 实例
    """
    global router_initialized

    model_config_manager = ModelConfigManager()
    agent_repo = SQLAgentRepository(agent_db)
    svc = LLMConfigService(model_config_manager, agent_repo)

    # 只有第一次进来时才注入
    if not router_initialized:
        init_llm_manager(svc)
        router_initialized = True

    return svc


@router.post("/model/{model_id}")
async def get_model(
    model_id: str,
    request: GetModelRequest,
    llm_service: LLMConfigService = Depends(get_llm_config_service),
    current_user: dict = Depends(get_current_user)
):
    """
    获取单个模型信息
    """
    _ = check_user_space(str(request.workspace_id), current_user)
    try:
        return await llm_service.get_model(model_id, request.model_from)
    except ValueError as e:
        raise HTTPException(status_code=404, detail=str(e)) from e
    except Exception as e:
        raise HTTPException(status_code=500, detail=str(e)) from e


@router.post("/models/list")
async def list_models(
    request: ListModelRequest,
    llm_service: LLMConfigService = Depends(get_llm_config_service),
    current_user: dict = Depends(get_current_user)
):
    """
    获取所有可用模型列表
    """
    _ = check_user_space(str(request.workspace_id), current_user)
    try:
        return await llm_service.list_models(request)
    except Exception as e:
        raise HTTPException(status_code=500, detail=str(e)) from e



from redis.asyncio import Redis as AsyncRedis
from redis import Redis as SyncRedis
from openjiuwen_studio.core.config import settings
from openjiuwen_studio.core.manager.model_manager.utils.security_utils import SecurityUtils


class RedisManager:
    """通用 Redis 管理器"""
    def __init__(self, decode_responses: bool = False, sync_client: bool = False):
        rds_password = SecurityUtils.get_decrypted_secret("REDIS_PASSWORD", settings.redis_password)
        if sync_client:
            self.client = SyncRedis(
                host=settings.redis_host,
                port=settings.redis_port,
                db=settings.redis_db,
                password=rds_password,
                decode_responses=decode_responses
            )
        else:
            self.client = AsyncRedis(
                host=settings.redis_host,
                port=settings.redis_port,
                db=settings.redis_db,
                password=rds_password,
                decode_responses=decode_responses
            )

    def set_ex(self, key: str, value: str, seconds: int):
        """设置带过期时间的缓存"""
        self.client.setex(key, seconds, value)

    def get(self, key: str):
        """获取缓存"""
        return self.client.get(key)

    def delete(self, key: str):
        """删除缓存"""
        self.client.delete(key)

    def exists(self, key: str) -> bool:
        """检查 Key 是否存在"""
        return self.client.exists(key)

    def incr(self, key: str, expire: int = None):
        """原子自增，可选设置过期时间"""
        count = self.client.incr(key)
        if count == 1 and expire:
            self.client.expire(key, expire)
        return count

redis_manager_bytes = RedisManager(decode_responses=False, sync_client=False)  # 异步客户端，返回 bytes
redis_manager_str = RedisManager(decode_responses=True, sync_client=True)      # 同步客户端，返回 str



#!/usr/bin/python3.10
# -*- coding: utf-8 -*-
# Copyright (c) Huawei Technologies Co., Ltd. 2025-2025. All rights reserved.

import asyncio


def observe(*args, **kwargs):
    """空接口，确保无sdk只是不上报trace,不影响执行主流程"""
    def decorator(func):
        # 如果是异步函数，确保正确处理
        if asyncio.iscoroutinefunction(func):
            async def async_wrapper(*args, **kwargs):
                return await func(*args, **kwargs)
            return async_wrapper
        return func
    return decorator


class NullSpan:

    @classmethod
    def __getattr__(cls, name):
        # 对于任何未定义的方法，返回一个空函数
        return lambda *args, **kwargs: None

    def set_attribute(self, key, value):
        """空接口，确保无sdk只是不上报trace,不影响执行主流程"""
        pass

    def set_input_tokens(self, tokens):
        """空接口，确保无sdk只是不上报trace,不影响执行主流程"""
        pass

    def set_prompt_key(self, key):
        """空接口，确保无sdk只是不上报trace,不影响执行主流程"""
        pass

    def set_prompt_version(self, version):
        """空接口，确保无sdk只是不上报trace,不影响执行主流程"""
        pass

    def set_space_id(self, space_id):
        """空接口，确保无sdk只是不上报trace,不影响执行主流程"""
        pass

    def set_stream(self, stream):
        """空接口，确保无sdk只是不上报trace,不影响执行主流程"""
        pass

    def set_input(self, input_data):
        """空接口，确保无sdk只是不上报trace,不影响执行主流程"""
        pass

    async def set_async_stream_output(self, async_gen_or_coroutine, **kwargs):
        """空接口，确保无sdk只是不上报trace,不影响执行主流程"""
        try:
            if asyncio.iscoroutine(async_gen_or_coroutine):
                # 这是一个协程，需要等待它返回生成器
                async_gen = await async_gen_or_coroutine
            else:
                # 这已经是一个异步生成器
                async_gen = async_gen_or_coroutine
            async for item in async_gen:
                yield item
        except Exception as e:
            self.error(e)
            raise


def set_baggage(*args, **kwargs):
    """空接口，确保无sdk只是不上报trace,不影响执行主流程"""
    pass


def get_baggage(*args, **kwargs):
    """空接口，确保无sdk只是不上报trace,不影响执行主流程"""
    return None


def calculate_input_tokens(*args, **kwargs):
    """空接口，确保无sdk只是不上报trace,不影响执行主流程"""
    return 0


def set_attribute(*args, **kwargs):
    """空接口，确保无sdk只是不上报trace,不影响执行主流程"""
    pass


def start_span(*args, **kwargs):
    """空接口，确保无sdk只是不上报trace,不影响执行主流程"""
    return NullSpan()


def end_span(*args, **kwargs):
    """空接口，确保无sdk只是不上报trace,不影响执行主流程"""
    pass


def inject(*args, **kwargs):
    """空接口，确保无sdk只是不上报trace,不影响执行主流程"""
    pass


class SpanType:
    Prompt = "prompt"
    PromptRunner = "prompt_runner"
    ToolCall = "tool_call"
    LLMCall = "llm_call"


class PlatformType:
    Prompt = "prompt"


class TraceModule:
    @classmethod
    def __call__(cls, *args, **kwargs):
        def decorator(func):
            return func
        return decorator

    @classmethod
    def get_current_span(cls):
        """空接口，确保无sdk只是不上报trace,不影响执行主流程"""
        return NullSpan()


# 创建 trace 实例
trace = TraceModule()



import os
from typing import Optional
from pydantic import Field
from pydantic_settings import BaseSettings


class Settings(BaseSettings):
    # Application
    app_name: str = "Jiuwen Agent Studio"
    app_version: str = "1.0.0"
    debug: bool = True

    # Server
    host: str = "0.0.0.0"
    port: int = 8000

    # LLM info for deepsearch
    llm_basic_base_url: str = ""
    llm_basic_model: str = ""
    llm_basic_api_key: str = ""
    llm_basic_api_type: str = ""

    # config path for deepsearch
    service_config_path: str = ""

    # 数据库类型配置 (mysql/sqlite)
    db_type: str = "mysql"

    # mysql配置
    db_host: str = ""
    db_port: int = 3306
    db_user: str = ""
    db_password: str = ""
    agent_db_name: str = ""

     # sqlite配置
    sqlite_db_path: str = "data/databases"
    agent_sqlite_db: str = "agent.db"

    # OBS配置(MinIO)
    minio_host: str = ""
    minio_port: int = 9000
    minio_access_key: str = ""
    minio_secret_key: str = ""
    minio_secure: bool = False
    minio_bucket: str = ""

    # deepsearch配置
    deepsearch_agent_host: str = ""
    deepsearch_agent_port: int = 6000

    # Security
    secret_key: str = "your-secret-key-change-in-production"
    algorithm: str = "HS256"
    # 单机版默认用户：设置很长的过期时间，避免用户被踢出
    # access_token: 10年，refresh_token: 10年
    access_token_expire_minutes: int = 5256000  # 10年 = 365 * 10 * 24 * 60
    refresh_token_expire_days: int = 3650  # 10年
    # 非单机部署使用较短的过期时间
    new_access_token_expire_minutes: int = 15 # 15分钟
    new_refresh_token_expire_days: int = 7 # 7天

    # CORS
    allowed_origins: list = [
        "http://localhost:3000",
        "http://127.0.0.1:3000"
    ]

    # Redis配置
    enable_redis_checkpoint: bool = False
    redis_host: str = "localhost"
    redis_port: int = 6379
    redis_db: int = 0
    redis_password: str = ""
    
    @property
    def redis_url(self) -> str:
        """构建 Redis 连接 URL"""
        if self.redis_password:
            return f"redis://:{self.redis_password}@{self.redis_host}:{self.redis_port}/{self.redis_db}"
        return f"redis://{self.redis_host}:{self.redis_port}/{self.redis_db}"
    
    # 云邮件推送
    smtp_host: str = ""
    smtp_port: int = 465
    smtp_user: str = ""
    smtp_password: str = ""
    smtp_alias: str = "openJiuwen"
    
    # 用户认证类型开关,只用在后端
    enable_new_auth: bool = Field(False, alias="VITE_ENABLE_NEW_AUTH")

    # External APIs
    openai_api_key: Optional[str] = None
    anthropic_api_key: Optional[str] = None

    class Config:
        env_file = "../.env"
        case_sensitive = False
        extra = "ignore"  # Ignore extra fields from .env file


# Create settings instance
settings = Settings()



from datetime import datetime
from enum import Enum
from functools import wraps
from typing import Callable

from fastapi import status
from openjiuwen.core.session.tracer.span import TraceWorkflowSpan
from sqlalchemy.orm import Session

from openjiuwen.core.common.logging import logger
from openjiuwen_studio.core.database import milliseconds
from openjiuwen_studio.core.manager.repositories import JiuwenBaseRepository
from openjiuwen_studio.core.manager.repositories.jiuwen_base_repository import (
    get_db_jw, get_val_from_dict)
from openjiuwen_studio.models.workflow import WorkflowBaseDB, WorkflowPublishDB
from openjiuwen_studio.models.workflow_execution import (WorkflowExecutionDB,
                                           WorkflowExecutionDetailsDB)
from openjiuwen_studio.schemas.common import ResponseModel
from openjiuwen_studio.schemas.execution_log import (ComponentExecuteStatus,
                                       ExecutionCallType,
                                       ExecutionLogCreateInfo,
                                       ExecutionLogDebug,
                                       ExecutionLogsCreateList,
                                       ExecutionLogSummary, InvokeExecuteInfo,
                                       WfExecutionLogIndex,
                                       WfExecutionLogsFilter)
from openjiuwen_studio.schemas.workflow import WorkflowId


def calc_duration_ms(start: datetime | None, end: datetime | None) -> int | None:
    return int((end - start).total_seconds() * 1000) if start and end else None


class WorkflowExecutionRepository():
    def __init__(self) -> None:
        # log中抽取至summary的共同字段, 这些字段只存于summary数据表，不会存于details数据表中
        self.common_fields_extract_from_log_to_summary = ["execution_id", "trace_id"]
        self.call_type_field = "call_type"          # 执行的发起者类型的属性名
        self.creat_time_field = 'create_time'       # 创建时间的属性名
        pass
    # def __init__(self, db: Session) -> None:
    #     self._workflow_execution_db: JiuwenBaseRepository[WorkflowExecutionDB] = JiuwenBaseRepository(db, WorkflowExecutionDB)
    #     self._workflow_execution_details_db: JiuwenBaseRepository[WorkflowExecutionDetailsDB] = JiuwenBaseRepository(db, WorkflowExecutionDetailsDB)

    def with_exception_handling(func) -> Callable:
        @wraps(func)
        def wrapper(self, *args, **kwargs):
            try:
                return func(self, *args, **kwargs)
            except Exception as e:
                error_log = f"Error: workflow execution db data preprocessing error, {type(e).__name__}"
                logger.error(error_log)
                return ResponseModel(code=status.HTTP_400_BAD_REQUEST, message=error_log)
        return wrapper
    
    @staticmethod
    def _extract_summary_execution_info_from_log(
        log_data: list[TraceWorkflowSpan],
        execute_start_time: datetime | None = None
    ) -> list[InvokeExecuteInfo] | None:
        """
        description: 从workflow的某次执行log_data中，提取总的执行信息, WfComponentsExecuteInfo类型
        """
        if not log_data:
            return None
        components_execute_info: list[InvokeExecuteInfo] = []       # 已处理好的组件执行信息
        running_component_execute_info: dict[str, InvokeExecuteInfo] = {}     # 当前还在处理中的组件
        if not execute_start_time:
            execute_start_time = log_data[0].start_time

        def _get_parent_component_info(invoke_id: str) -> tuple[InvokeExecuteInfo | None, str | None]:
            """
            description: 根据invoke_id获取父组件信息
            invoke_id规则：
            - workflow_cLjMT: 子工作流
            - workflow_cLjMT.start_0: 该工作流的start子组件
            - workflow_cLjMT.workflow_fV0Wb: workflow_cLjMT执行了其子工作流workflow_fV0Wb
            - loop_U2lTa: 循环节点
            - loop_U2lTa.workflow_hjFr6: 循环节点下的子工作流
            - loop_U2lTa.workflow_hjFr6.end_0: 子工作流的end节点
            """
            if not invoke_id:
                return None, None
            
            # 检查是否为工作流、循环节点或其子组件
            if invoke_id.startswith(('workflow_', 'loop_')):
                # 分割invoke_id获取父级关系
                parts = invoke_id.split('.')
                
                # 对于workflow_cLjMT.start_0，父级是workflow_cLjMT
                # 对于loop_U2lTa.workflow_hjFr6，父级是loop_U2lTa
                # 对于loop_U2lTa.workflow_hjFr6.end_0，父级是loop_U2lTa.workflow_hjFr6
                if len(parts) > 1:
                    parent_type = '.'.join(parts[:-1])
                    if parent_type in running_component_execute_info:
                        return running_component_execute_info[parent_type], parent_type
            
            return None, None

        def _component_process_over(log: TraceWorkflowSpan | None, component_execute_info: InvokeExecuteInfo):
            """
            description: 结束running组件的处理过程
            """
            if log:
                component_execute_info.status = log.status
                component_execute_info.duration = calc_duration_ms(log.start_time, log.end_time)
                component_execute_info.outputs = log.outputs
            else:   # 没有对应的log，说明此component没有正常结束, 可能是被终止了
                component_execute_info.status = ComponentExecuteStatus.error
            
            # 1. 优先处理循环组件关系
            if component_execute_info.loop_node_id: 
                # 处理循环组件中的子组件
                if component_execute_info.loop_node_id in running_component_execute_info:
                    # 将子组件的执行信息添加到父组件中
                    parent_component_execute_info = running_component_execute_info[component_execute_info.loop_node_id]
                    if parent_component_execute_info.child_invokes_execute_info is None:
                        parent_component_execute_info.child_invokes_execute_info = []
                    parent_component_execute_info.child_invokes_execute_info.append(component_execute_info)
                    return
                else:
                    # 还处理中的组件中无法找到父节点，说明存在错误
                    error_msg = "Can't find the parent loop_component of this component"
                    logger.debug(error_msg)
                    # 不抛出异常，继续尝试其他父级关系
            
            # 2. 处理子工作流/子组件的层级关系
            parent_info, parent_type = _get_parent_component_info(component_execute_info.invoke_id)
            if parent_info:
                # 将当前组件作为父组件的子组件
                if parent_info.child_invokes_execute_info is None:
                    parent_info.child_invokes_execute_info = []
                parent_info.child_invokes_execute_info.append(component_execute_info)
                return
            

            
            # 4. 一般组件，直接添加到结果列表
            components_execute_info.append(component_execute_info)

        for log in log_data:
            # 统计单个组件的执行信息
            # 获取组件名称
            component_name = log.component_name
            
            # 组件开始
            if log.status == ComponentExecuteStatus.start:
                component_execute_info = InvokeExecuteInfo(
                    invoke_id=log.invoke_id, 
                    invoke_name=component_name,
                    invoke_type=log.component_type,
                    status=log.status,
                    start_timestamp=calc_duration_ms(execute_start_time, log.start_time),
                    loop_node_id=log.loop_node_id,
                    loop_index=log.loop_index,
                    inputs=log.inputs,
                )
                # 保存到running字典，key使用invoke_id
                # 对于工作流和循环节点，使用完整的invoke_id以便后续根据层级关系查找
                running_component_execute_info[log.invoke_id] = component_execute_info
            # 组件结束 或 报错
            else:
                # 从正处理的组件中取出该组件信息
                component_execute_info = running_component_execute_info.pop(log.invoke_id, None)
                if component_execute_info:
                    _component_process_over(log, component_execute_info)
                else:
                    warn_msg = f"Can't find this component from running components, invoke_id: {log.invoke_id}"
                    logger.warning(warn_msg)
                    component_execute_info = InvokeExecuteInfo(
                        invoke_id=log.invoke_id,
                        invoke_name=component_name,
                        invoke_type=log.component_type,
                        status=ComponentExecuteStatus.start,
                        start_timestamp=calc_duration_ms(execute_start_time, log.start_time),
                        loop_node_id=log.loop_node_id,
                        loop_index=log.loop_index,
                        inputs=log.inputs,
                    )
                    _component_process_over(log, component_execute_info)
        # 处理剩余的正处理中的组件，说明这些组件没有正常结束
        while running_component_execute_info:
            invoke_id, component_execute_info = running_component_execute_info.popitem()
            _component_process_over(None, component_execute_info)
        return components_execute_info
    
    def _summary_log_data(self, workflow: WorkflowBaseDB | WorkflowPublishDB, log_data: list[TraceWorkflowSpan], 
                          execute_start_time: datetime | None = None) -> WorkflowExecutionDB:
        """
        description: 创建 WorkflowExecutionDB 数据
        """
        # 本次workflow执行的所有component执行数据
        components_execute_info = WorkflowExecutionRepository._extract_summary_execution_info_from_log(log_data=log_data,
                                                                                execute_start_time=execute_start_time)
        # 将workflow的本次执行的总信息构造成最外层的InvokeExecuteInfo
        execute_info = InvokeExecuteInfo(
            invoke_id=workflow.workflow_id,
            invoke_version=workflow.workflow_version,
            invoke_type=ExecutionCallType.workflow,
            invoke_name=workflow.name,
            start_timestamp=0 if not execute_start_time else calc_duration_ms(
                execute_start_time, log_data[0].start_time),
            duration=calc_duration_ms(log_data[0].start_time, log_data[-1].end_time),
            inputs=log_data[0].inputs,
        )
        if components_execute_info:
            execute_info.status = components_execute_info[-1].status
            execute_info.outputs = components_execute_info[-1].outputs
            execute_info.child_invokes_execute_info = components_execute_info
        else:
            execute_info.status = ComponentExecuteStatus.error
        # 构造保存到db的数据
        summary_db = WorkflowExecutionDB(
            space_id=workflow.space_id,
            workflow_id=workflow.workflow_id,
            workflow_version=workflow.workflow_version,
            trace_id=log_data[0].trace_id,
            status=execute_info.status,
            execution_id=log_data[0].execution_id,
            duration=execute_info.duration,
            inputs=execute_info.inputs,
            outputs=execute_info.outputs,
            create_time=log_data[0].start_time,
            update_time=log_data[-1].end_time if log_data[-1].end_time else log_data[-1].start_time,
        )
        summary_db.execute_info_list = [execute_info.model_dump(exclude_none=True)]
        if summary_db.status == ComponentExecuteStatus.error:
            if isinstance(log_data[-1].error, dict):
                _code = log_data[-1].error.get("code", None)
                try:
                    summary_db.error_code = int(_code) if _code is not None else None
                except (TypeError, ValueError):
                    summary_db.error_code = None
                summary_db.fail_reason = str(log_data[-1].error.get("message", "unknown"))
            else:
                summary_db.error_code = None
                summary_db.fail_reason = str(log_data[-1].error)
        return summary_db
    
    def _log_data_2_details_db(self, log_data: list[TraceWorkflowSpan]) -> list[WorkflowExecutionDetailsDB]:
        """
        description: 创建 WorkflowExecutionDetailsDB 数据
        """
        detail_dbs = []
        for log in log_data:
            log_dict = log.model_dump(exclude=self.common_fields_extract_from_log_to_summary)
            # 确保 error 字段是字典类型或 None
            if 'error' in log_dict and log_dict['error'] is not None and not isinstance(log_dict['error'], dict):
                # 将非字典类型的错误转换为字典格式
                log_dict['error'] = {'message': str(log_dict['error'])}
            detail_dbs.append(WorkflowExecutionDetailsDB.from_dict(log_dict))
        return detail_dbs
    
    '''
    description: 创建WorkflowExecutionDB模型的数据
    param {*} self
    param {WorkflowId} workflow_index   本次执行的workflow信息
    param {list} log_data       本次执行的日志信息列表， list[TraceWorkflowSpan]
    param {ExecutionCallType} call_type 本次执行的调用类型，默认为workflow
    param {int} agent_execution_detail_id   本次workflow由agent发起调用时，agent的agent_execution_details.id; 
                                            如果是通过sqlalchemy的级联创建，不需要赋值agent_execution_detail_id
    param {int} agent_execution_detail_id   本次执行的真正开始时间, 如果workflow是某执行的子项, 需要输入此值作为执行的开始时间, 而不是从wf中提取; 
    return {*}      WorkflowExecutionDB模型的数据放在ResponseModel.data中
    '''

    def generate_workflow_execution_dbmodel(
        self, dbsession: Session, workflow_index: WorkflowId,
        log_data: list[TraceWorkflowSpan],
        call_type: ExecutionCallType = ExecutionCallType.workflow,
        agent_execution_detail_id: int | None = None,
        execute_start_time: datetime | None = None
    ) -> ResponseModel[WorkflowExecutionDB | None]:
        # 数据校验
        if not log_data:
            logger.debug("No log_data to register")
            return ResponseModel(
                code=status.HTTP_400_BAD_REQUEST, 
                message="No log_data to register"
            )
        
        # 对于log_data, 可能最后一个数据非TraceWorkflowSpan类型而是outputstream类型，所以需要先剔除
        if not isinstance(log_data[-1], TraceWorkflowSpan):
            log_data = log_data[:-1]

        # trace_id唯一性及有效性校验
        trace_id_set = list(set([log.trace_id for log in log_data]))
        trace_id = trace_id_set[0]
        if len(trace_id_set) != 1 or not trace_id:
            logger.debug(f"Trace_id in log_data is not unique or empty, trace_id_set: {trace_id_set}")
            return ResponseModel(
                code=status.HTTP_400_BAD_REQUEST,
                message="Trace_id in log_data is not unique or empty."
            )
        # execution_id唯一性校验
        execution_id_set = list(set([log.execution_id for log in log_data]))
        if len(execution_id_set) != 1:
            logger.debug(f"Execution_id in log_data is not unique, execution_id_set: {execution_id_set}.")
            return ResponseModel(
                code=status.HTTP_400_BAD_REQUEST,
                message="Execution_id in log_data is not unique."
            )
        
        # 数据库查询workflow是否存在
        if not workflow_index.workflow_version or workflow_index.workflow_version == WorkflowBaseDB.__version_none__:
            # 非发布版本的workflow
            workflow_db = JiuwenBaseRepository(dbsession, WorkflowBaseDB)
            workflow_index.workflow_version = WorkflowBaseDB.__version_none__
        else:
            workflow_db = JiuwenBaseRepository(dbsession, WorkflowPublishDB)
        # 先查找workflow是否存在
        workflow_db_res = workflow_db.get_dl_in_sql(find_id=workflow_index.model_dump(), 
                                                     return_first_item=True, return_declarativebase=True)
        if workflow_db_res.code != status.HTTP_200_OK:
            return workflow_db_res
        
        # 构造summary信息
        execution_summary_create: WorkflowExecutionDB = self._summary_log_data(workflow_db_res.data, log_data, 
                                                                              execute_start_time)
        execution_summary_create.call_type = call_type
        if call_type == ExecutionCallType.agent and agent_execution_detail_id:
            # 如果本次workflow是由agent发起调用; 如果是通过sqlalchemy的级联创建，可以不用赋值agent_execution_detail_id
            execution_summary_create.agent_execution_detail_id = agent_execution_detail_id
        
        # 再构造详细信息
        execution_summary_create.workflow_execution_details_list += self._log_data_2_details_db(log_data)

        # 返回数据
        return ResponseModel(code=status.HTTP_200_OK, message="Generate workflow execution dbmodel ok.", data=execution_summary_create)
        
    @with_exception_handling
    def create_workflow_execution_log(self, workflow_index: WorkflowId, log_data: list[TraceWorkflowSpan],
                                       call_type: ExecutionCallType = ExecutionCallType.workflow, 
                                       agent_execution_detail_id: int | None = None, 
                                       db_session: Session | None = None, 
                                       execute_start_time: datetime | None = None) -> ResponseModel[None]:
        with get_db_jw(db_session) as db:
            wf_exe_dbmodel_res = self.generate_workflow_execution_dbmodel(
                db, workflow_index, log_data, call_type,
                agent_execution_detail_id, execute_start_time
            )
            if isinstance(wf_exe_dbmodel_res.data, WorkflowExecutionDB):
                workflow_execution_db = JiuwenBaseRepository(db, WorkflowExecutionDB)
                return workflow_execution_db.register_dl_in_sql(find_id=None, dl=wf_exe_dbmodel_res.data)
            return wf_exe_dbmodel_res
        
    '''
    description: 删除workflow的所有执行日志
    param {*} self
    param {WorkflowId} workflow_index
    return {*}
    '''
    @with_exception_handling
    def delete_all_execution_logs_of_workflow(self, workflow_index: WorkflowId,
                                       call_type: ExecutionCallType | None = ExecutionCallType.workflow, 
                                       db_session: Session | None = None) -> ResponseModel[None]:
        # 数据校验
        if not workflow_index.workflow_version:
            workflow_index.workflow_version = WorkflowExecutionDB.__version_none__
        find_id = workflow_index.model_dump()
        if call_type:
            find_id[self.call_type_field] = call_type.value
        with get_db_jw(db_session) as db:
            workflow_execution_db = JiuwenBaseRepository(db, WorkflowExecutionDB)
            return workflow_execution_db.unregister_dl_in_sql(find_id=find_id)
    
    '''
    description: 删除workflow的某次执行日志
    param {*} self
    param {WorkflowId} workflow_index
    return {*}
    '''
    @with_exception_handling
    def delete_workflow_execution_log(self, wf_execution_log_index: WfExecutionLogIndex, 
                                      db_session: Session | None = None) -> ResponseModel[None]:
        # 数据校验
        if not wf_execution_log_index.workflow_version:
            wf_execution_log_index.workflow_version = WorkflowExecutionDB.__version_none__
        with get_db_jw(db_session) as db:
            workflow_execution_db = JiuwenBaseRepository(db, WorkflowExecutionDB)
            return workflow_execution_db.unregister_dl_in_sql(find_id=wf_execution_log_index.model_dump())

    '''
    description: 获取一定范围内所有执行日志的创建list
    param {*} self
    param {WfExecutionLogsFilter} wf_execution_logs_filter
    param {*} dbsession     传入的db session
    return {*}
    '''

    def _get_execution_logs_create_list_with_dbsession(
        self, dbsession: Session, wf_execution_logs_filter: WfExecutionLogsFilter,
        call_type: ExecutionCallType | None = ExecutionCallType.workflow
    ) -> ResponseModel[ExecutionLogDebug]:
        if not wf_execution_logs_filter.workflow_version:
            wf_execution_logs_filter.workflow_version = WorkflowExecutionDB.__version_none__
        wf_execution_logs_filter = WfExecutionLogsFilter(**wf_execution_logs_filter.model_dump())
        find_id = wf_execution_logs_filter.model_dump(exclude_none=True)
        if call_type:
            find_id[self.call_type_field] = call_type.value
        # 需要从数据库中获取的字段
        cols_find = list(ExecutionLogCreateInfo.model_fields.keys())
        find_min_max = {self.creat_time_field: [wf_execution_logs_filter.start_time, wf_execution_logs_filter.end_time]}
        order_cols_desc = [self.creat_time_field]   # 降序排列
        workflow_execution_db = JiuwenBaseRepository(dbsession, WorkflowExecutionDB)
        db_res = workflow_execution_db.get_dl_in_sql_with_cols(find_id=find_id, cols_find=cols_find,
                                                            find_min_max=find_min_max, order_cols_desc=order_cols_desc)
        if db_res.code == status.HTTP_200_OK and db_res.data:
            logs_create_list = [ExecutionLogCreateInfo(**record) for record in db_res.data]
            db_res.data = ExecutionLogDebug(logs_create_list=logs_create_list)
        return db_res

    @with_exception_handling
    def get_execution_logs_create_list(self, wf_execution_logs_filter: WfExecutionLogsFilter,
                                       call_type: ExecutionCallType | None = ExecutionCallType.workflow, 
                                       db_session: Session | None = None) -> ResponseModel[ExecutionLogDebug]:
        with get_db_jw(db_session) as db:
            return self._get_execution_logs_create_list_with_dbsession(db, wf_execution_logs_filter, call_type=call_type)

    '''
    description: 获取workflow的某次执行日志详情
    param {*} self
    param {WfExecutionLogIndex} wf_execution_log_index
    param {*} dbsession     传入的db session
    return {*}
    '''

    def get_execution_log_with_dbsession(
        self, dbsession: Session, wf_execution_log_index: WfExecutionLogIndex
    ) -> ResponseModel[ExecutionLogDebug]:
        if not wf_execution_log_index.workflow_version:
            wf_execution_log_index.workflow_version = WorkflowExecutionDB.__version_none__
        workflow_execution_db = JiuwenBaseRepository(dbsession, WorkflowExecutionDB)
        db_res = workflow_execution_db.get_dl_in_sql(find_id=wf_execution_log_index.model_dump(), return_first_item=True,
                                                        return_declarativebase=True)
        if db_res.code != status.HTTP_200_OK:
            return db_res
        log_summary = ExecutionLogSummary.model_validate(db_res.data.to_dict())
        # 获取workflow执行信息中的共同字段
        details_common_fields_dict = {field: getattr(db_res.data, field)
                                                     for field in self.common_fields_extract_from_log_to_summary}
        workflow_execution_details_list = db_res.data.workflow_execution_details_list
        log_details = [TraceWorkflowSpan.model_validate(detail.to_dict() | details_common_fields_dict) 
                                                for detail in workflow_execution_details_list]
        db_res.data = ExecutionLogDebug(log_summary=log_summary, log_details=log_details)
        return db_res
    
    @with_exception_handling
    def get_execution_log(self, wf_execution_log_index: WfExecutionLogIndex, db_session: Session | None = None) -> ResponseModel[ExecutionLogDebug]:
        with get_db_jw(db_session) as db:
            return self.get_execution_log_with_dbsession(db, wf_execution_log_index)

    '''
    description: 进行调试日志界面时，返回所有日志create list和最新日志详情
    param {*} self
    param {WorkflowId} workflow_id
    return {*}
    '''
    @with_exception_handling
    def enter_execution_logs_debug(self, workflow_id: WorkflowId, call_type: ExecutionCallType | None = ExecutionCallType.workflow, 
                                   db_session: Session | None = None) -> ResponseModel[ExecutionLogDebug]:
        with get_db_jw(db_session) as db:
            # 获取workflow的运行create list
            logs_create_list_db_res = self._get_execution_logs_create_list_with_dbsession(db, workflow_id,  
                                                                                          call_type=call_type)
            if logs_create_list_db_res.code != status.HTTP_200_OK or not logs_create_list_db_res.data \
                        or not logs_create_list_db_res.data.logs_create_list:
                return logs_create_list_db_res
            # 获取最新运行日志的数据
            logs_create_list: list[ExecutionLogCreateInfo] = logs_create_list_db_res.data.logs_create_list
            lastest_trace_id = logs_create_list[0].trace_id
            wf_execution_log_index = WfExecutionLogIndex.model_validate(
                workflow_id.model_dump() | {"trace_id": lastest_trace_id})
            laster_execution_log_db_res = self.get_execution_log_with_dbsession(db, wf_execution_log_index)
            if laster_execution_log_db_res.code != status.HTTP_200_OK:
                return laster_execution_log_db_res
            laster_execution_log_db_res.data.logs_create_list = logs_create_list
            return laster_execution_log_db_res


workflow_execution_repository = WorkflowExecutionRepository()


"""fix mysql agent update

Revision ID: 072ac1293a02
Revises: 54be202e347e
Create Date: 2026-02-07 11:16:02.133215

"""
from typing import Sequence, Union

from alembic import op
from openjiuwen_studio.core.database.migration_utils import table_exists, column_exists, index_exists
import sqlalchemy as sa
from sqlalchemy.dialects import mysql
from sqlalchemy import inspect

# revision identifiers, used by Alembic.
revision: str = '072ac1293a02'
down_revision: Union[str, Sequence[str], None] = '54be202e347e'
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Upgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    if not table_exists('memory_base'):
        op.create_table('memory_base',
        sa.Column('id', sa.BigInteger(), autoincrement=True, nullable=False),
        sa.Column('space_id', sa.String(length=100), nullable=False,
                     comment='空间ID，用于多租户隔离'),
        sa.Column('mdb_id', sa.String(length=100), nullable=False, comment='记忆库ID，唯一标识'),
        sa.Column('name', sa.String(length=100), nullable=False, comment='记忆库名称'),
        sa.Column('description', sa.String(length=100), nullable=False, comment='记忆库名描述'),
        sa.Column('embedding_model_config_id', sa.Integer(), nullable=False,
                     comment='Embedding 模型配置ID'),
        sa.Column('llm_model_config_id', sa.Integer(), nullable=False,
                     comment='LLM 模型配置ID'),
        sa.Column('create_time', sa.BigInteger(), nullable=False, comment='创建时间'),
        sa.Column('update_time', sa.BigInteger(), nullable=False, comment='更新时间'),
        sa.PrimaryKeyConstraint('id'),
        sa.UniqueConstraint('mdb_id', name='uix_mdb_id'),
        comment='记忆库表，存储记忆库基本信息'
        )

    if not index_exists('memory_base', 'idx_space_id'):
        op.create_index('idx_space_id', 'memory_base', ['space_id'], unique=False)
    if not index_exists('memory_base', 'idx_space_mdb'):
        op.create_index('idx_space_mdb', 'memory_base', ['space_id', 'mdb_id'], unique=False)
    if not index_exists('memory_base', 'ix_memory_base_embedding_model_config_id'):
        op.create_index(op.f('ix_memory_base_embedding_model_config_id'),
                     'memory_base', ['embedding_model_config_id'], unique=False)
    if not index_exists('memory_base', 'ix_memory_base_llm_model_config_id'):
        op.create_index(op.f('ix_memory_base_llm_model_config_id'),
                     'memory_base', ['llm_model_config_id'], unique=False)
    if not index_exists('memory_base', 'ix_memory_base_mdb_id'):
        op.create_index(op.f('ix_memory_base_mdb_id'), 'memory_base', ['mdb_id'], unique=False)

    if not column_exists('knowledge_base', 'index_manager_type'):
        op.add_column('knowledge_base',
                   sa.Column('index_manager_type', sa.String(length=200),
                             nullable=True,
                             comment='Whether Chroma or Milvus is used for indexing'))
    if not column_exists('knowledge_base_document', 'index_manager_type'):
        op.add_column('knowledge_base_document',
                   sa.Column('index_manager_type', sa.String(length=200),
                             nullable=True,
                             comment='Whether Chroma or Milvus is used for indexing'))
    if not column_exists('knowledge_base_document', 'index_id'):
        op.add_column('knowledge_base_document',
                   sa.Column('index_id', sa.String(length=200), nullable=True,
                             comment='ES中的索引ID'))
    if not column_exists('knowledge_base_document', 'index_name'):
        op.add_column('knowledge_base_document',
                   sa.Column('index_name', sa.String(length=200), nullable=True,
                             comment='ES索引名称'))
    
    if column_exists('knowledge_base_document', 'es_index_name'):
        op.drop_column('knowledge_base_document', 'es_index_name')
    if column_exists('knowledge_base_document', 'es_index_id'):
        op.drop_column('knowledge_base_document', 'es_index_id')
    # ### end Alembic commands ###


def downgrade() -> None:
    """Downgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    if not column_exists('knowledge_base_document', 'es_index_id'):
        op.add_column('knowledge_base_document',
                   sa.Column('es_index_id',
                             mysql.VARCHAR(collation='utf8mb4_unicode_ci', length=200),
                             nullable=True, comment='ES中的索引ID'))
    if not column_exists('knowledge_base_document', 'es_index_name'):
        op.add_column('knowledge_base_document',
                   sa.Column('es_index_name',
                             mysql.VARCHAR(collation='utf8mb4_unicode_ci', length=200),
                             nullable=True, comment='ES索引名称'))
    
    if column_exists('knowledge_base_document', 'index_name'):
        op.drop_column('knowledge_base_document', 'index_name')
    if column_exists('knowledge_base_document', 'index_id'):
        op.drop_column('knowledge_base_document', 'index_id')
    if column_exists('knowledge_base_document', 'index_manager_type'):
        op.drop_column('knowledge_base_document', 'index_manager_type')
    if column_exists('knowledge_base', 'index_manager_type'):
        op.drop_column('knowledge_base', 'index_manager_type')

    if index_exists('memory_base', 'ix_memory_base_mdb_id'):
        op.drop_index(op.f('ix_memory_base_mdb_id'), table_name='memory_base')
    if index_exists('memory_base', 'ix_memory_base_llm_model_config_id'):
        op.drop_index(op.f('ix_memory_base_llm_model_config_id'), table_name='memory_base')
    if index_exists('memory_base', 'ix_memory_base_embedding_model_config_id'):
        op.drop_index(op.f('ix_memory_base_embedding_model_config_id'), table_name='memory_base')
    if index_exists('memory_base', 'idx_space_mdb'):
        op.drop_index('idx_space_mdb', table_name='memory_base')
    if index_exists('memory_base', 'idx_space_id'):
        op.drop_index('idx_space_id', table_name='memory_base')
    
    if table_exists('memory_base'):
        op.drop_table('memory_base')
    # ### end Alembic commands ###



from typing import List

from fastapi import HTTPException, Depends, status

from openjiuwen.core.common.logging import logger

from openjiuwen_studio.core.manager.repositories.user_repository import user_repository
from openjiuwen_studio.core.manager.dfx.status import get_space_status_by_id
from openjiuwen_studio.core.manager.login_manager.user import get_user_id
from openjiuwen_studio.schemas.space import SpaceInfo, SpaceDBPd, SpaceBase


def create_space_info(space_db: SpaceDBPd) -> SpaceInfo:
    # 获取 SpaceDBPd 的基础字段
    base_fields = set(SpaceBase.__fields__.keys())
    base_data = space_db.dict(include=base_fields)

    # 添加 SpaceInfo 的特有字段（使用默认值）
    return SpaceInfo(**base_data)


def get_space_by_user_id(user_id: str) -> List[SpaceInfo]:
    ret = user_repository.get_space_id(user_id)
    if ret["code"] != status.HTTP_200_OK:
        raise HTTPException(status_code=404, detail="space id not found")
    space_list = ret.get("data", [])
    space_info_list = []
    
    # 容错处理：如果某个 space 查询失败，记录警告但继续处理其他 space
    for space_id in space_list:
        ret = user_repository.get_space_tbl(space_id)
        if ret["code"] != status.HTTP_200_OK:
            error_msg = ret.get('message', 'unknown error')
            error_code = ret.get('code')
            logger.warning(
                f"[GET_SPACE_BY_USER] Failed to get space {space_id} for user {user_id}: "
                f"{error_msg}, code: {error_code}"
            )
            continue  # 跳过失败的 space，继续处理其他的
        try:
            space_db = ret["data"]
            space_info = create_space_info(SpaceDBPd(**space_db))
            space_info_list.append(space_info)
        except Exception as e:
            logger.warning(
                f"[GET_SPACE_BY_USER] Failed to create space info for space {space_id}: {str(e)}"
            )
            continue  # 跳过创建失败的 space，继续处理其他的

    if not space_info_list:
        logger.error(f"[GET_SPACE_BY_USER] No valid spaces found for user {user_id} from space_list: {space_list}")
        raise HTTPException(status_code=404, detail="No valid spaces found for user")
    
    return space_info_list


def find_space_by_id(space_id: str, space_info_list: List[SpaceInfo]) -> SpaceInfo:
    """根据空间ID在空间列表中查找特定空间"""
    for space in space_info_list:
        if space_id == space.space_id:
            return space
    raise HTTPException(status_code=404, detail=f"User doesn't have space id: {space_id}")


def get_space_list(current_user: dict) -> List[SpaceInfo]:
    """Get all spaces by user ID get from session token"""
    try:
        user_id = get_user_id(current_user)
        if user_id is None:
            raise HTTPException(status_code=404, detail="user id not found")

        return get_space_by_user_id(user_id)

    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"Failed to retrieve space list: {str(e)}")
        raise HTTPException(status_code=500, detail="Failed to retrieve space list") from e


def check_user_space(space_id: str, current_user: dict) -> SpaceInfo:
    """check the access by space id"""
    try:
        space_info_list = get_space_list(current_user)
        space_info = None
        for space in space_info_list:
            if space_id == space.space_id:
                space_info = space
                break
        if space_info is None:
            raise HTTPException(status_code=status.HTTP_403_FORBIDDEN, detail=f"space_id {space_id} Access Denied")
        get_space_status_by_id(space_info, space_id)
        return space_info

    except HTTPException:
        raise
    except Exception as e:
        logger.error(f"Failed to check user space: {str(e)}")
        raise HTTPException(status_code=500, detail="Internal server error") from e



#!/usr/bin/python3.10
# coding: utf-8
# Copyright (c) Huawei Technologies Co., Ltd. 2025-2025. All rights reserved
"""exception"""


class NotFoundException(Exception):
    """资源未找到异常"""
    pass


class RelationException(Exception):
    """资源关联异常"""
    pass


class DuplicateException(Exception):
    """重复资源异常"""
    pass


class ValidationException(Exception):
    """数据验证异常"""
    pass


class UnauthorizedException(Exception):
    """未授权异常"""
    pass


class ForbiddenException(Exception):
    """禁止访问异常"""
    pass


class ServiceException(Exception):
    """服务层异常基类"""
    pass


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (c) Huawei Technologies Co., Ltd. 2025-2025. All rights reserved.
import sys
import importlib.util
from pathlib import Path
# Import migration utilities
from .migration_utils import table_exists, column_exists, index_exists

# Import from database.py module to maintain backward compatibility

# Add parent directory to path to import database.py
_db_module_path = Path(__file__).parent.parent / 'database.py'
spec = importlib.util.spec_from_file_location("openjiuwen_studio.core.database._module", _db_module_path)
_db_module = importlib.util.module_from_spec(spec)
spec.loader.exec_module(_db_module)

# Re-export everything from database.py
for attr in dir(_db_module):
    if not attr.startswith('_'):
        globals()[attr] = getattr(_db_module, attr)

__all__ = ['table_exists', 'column_exists', 'index_exists']



import time
from pathlib import Path
from urllib3 import PoolManager, Timeout

from minio import Minio
from minio.error import S3Error
from minio.commonconfig import ENABLED, Filter
from minio.lifecycleconfig import LifecycleConfig, Rule, Expiration

from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker

from openjiuwen.core.common.logging import logger
from openjiuwen_studio.core.config import settings
from openjiuwen_studio.core.manager.model_manager.utils.security_utils import SecurityUtils


def get_database_url() -> str:
    """根据数据库类型生成数据库连接URL"""
    if settings.db_type.lower() == "mysql":
        db_password = SecurityUtils.get_decrypted_secret("DB_PASSWORD", settings.db_password)
        return (f"mysql+pymysql://{settings.db_user}:{db_password}@"
                   f"{settings.db_host}:{settings.db_port}/{settings.agent_db_name}?charset=utf8mb4")

    elif settings.db_type.lower() == "sqlite":
        # 确保数据库目录存在
        db_path = Path(settings.sqlite_db_path)
        db_path.mkdir(parents=True, exist_ok=True)
        return f"sqlite:///{db_path}/{settings.agent_sqlite_db}"

    else:
        raise ValueError(f"Unsupported database type: {settings.db_type.lower()}")

database_url = get_database_url()

# Create database engine
engine = create_engine(
    database_url,
    connect_args={"check_same_thread": False} if "sqlite" in database_url else {}
)

# Create session factory
SessionLocal = sessionmaker(autocommit=False, autoflush=False, bind=engine)


# Dependency to get database session
def get_db():
    db = SessionLocal()
    try:
        yield db
    finally:
        db.close()


def get_milliseconds() -> int:
    """返回当前时间戳的毫秒整数部分."""
    return int(time.time() * 1000)


milliseconds = get_milliseconds


class LazyMinioClient:
    """
    实现 MinIO 客户端的懒加载和错误处理。
    只在首次需要时尝试初始化，如果初始化失败则记录错误并保持未初始化状态，
    后续每次操作都尝试重新初始化，失败则抛出异常。
    """
    _instance = None
    _client = None
    _initialized = False

    def __new__(cls):
        if cls._instance is None:
            cls._instance = super(LazyMinioClient, cls).__new__(cls)
        return cls._instance

    @staticmethod
    def ensure_minio_lifecycle(minio_client: Minio, bucket_name: str, days: int = 99):
        """
        确保 MinIO bucket 设置了自动过期规则（days 天后删除）
        使URL永久有效（在文件存在期间），但文件会自动过期删除
        只需调用一次，幂等安全。
        """
        try:
            if not minio_client.bucket_exists(bucket_name):
                minio_client.make_bucket(bucket_name)
                logger.info(f"Bucket '{bucket_name}' created.")
            # 构造生命周期规则
            config = LifecycleConfig(
                [
                    Rule(
                        ENABLED,
                        rule_filter=Filter(prefix=""),
                        expiration=Expiration(days=days)
                    )
                ]
            )
            minio_client.set_bucket_lifecycle(bucket_name, config)
            # 设置桶策略为公开读（URL永久有效直到文件被删除）
            import json
            policy = {
                "Version": "2012-10-17",
                "Statement": [
                    {
                        "Effect": "Allow",
                        "Principal": {"AWS": "*"},
                        "Action": ["s3:GetObject"],
                        "Resource": [f"arn:aws:s3:::{bucket_name}/*"]
                    }
                ]
            }
            minio_client.set_bucket_policy(bucket_name, json.dumps(policy))
            logger.info(f"Bucket '{bucket_name}' configured: lifecycle {days} days, public read enabled.")
        except Exception as e:
            logger.exception(f"Failed to set lifecycle rule for bucket '{bucket_name}': {e}")

    def get_client(self) -> Minio:
        """获取 MinIO 客户端实例，如果尚未初始化或上次初始化失败，则尝试初始化。"""
        if not self._initialized:
            self._initialize_client()
        if not self._client:
            raise RuntimeError("MinIO client is not available due to "
                               "previous initialization failure or missing configuration.")
        return self._client

    def _initialize_client(self):
        """初始化 MinIO 客户端"""
        try:
            logger.debug("Attempting to initialize MinIO client...")
            # 尝试从 settings 中获取配置
            if not all([settings.minio_host, settings.minio_port,
                        settings.minio_access_key, settings.minio_secret_key]):
                raise ValueError("One or more required MinIO configuration settings are missing.")
            # 配置带超时的 HTTP 客户端
            http_client = PoolManager(
                timeout=Timeout(connect=3.0, read=5.0),
                retries=False
            )
            client = Minio(
                endpoint=f"{settings.minio_host}:{settings.minio_port}",
                access_key=settings.minio_access_key,
                secret_key=settings.minio_secret_key,
                secure=settings.minio_secure,
                http_client=http_client
            )

            # 进行一个简单的操作验证连接
            buckets = client.list_buckets()

            self._client = client
            self.ensure_minio_lifecycle(client, settings.minio_bucket)
            self._initialized = True
            logger.info("MinIO client initialized successfully.")
        except (ValueError, S3Error) as e:
            self._client = None
            self._initialized = False # 确保标记为未初始化
            logger.error(f"Failed to initialize MinIO client: {e}. Service will continue without MinIO.")
        except Exception as e:
            self._client = None
            self._initialized = False
            logger.exception(f"Unexpected error during MinIO client initialization: {e}. "
                                   f"Service will continue without MinIO.")


# 提供一个便捷函数给外部使用
def get_minio_client() -> Minio:
    """便捷函数，获取懒加载的 MinIO 客户端实例。"""
    lazy_client = LazyMinioClient()
    return lazy_client.get_client()



from sqlalchemy import BigInteger, String, UniqueConstraint, Integer
from sqlalchemy.orm import Mapped, mapped_column
from openjiuwen_studio.models.db_fun_base import Base, DBFunBase
from openjiuwen_studio.ops.config import settings


class ReferenceDB(Base, DBFunBase):
    __tablename__ = "reference"
    __table_args__ = (
        UniqueConstraint("space_id", "referenced_type", "referenced_id", "referenced_version",
                        "referer_type", "referer_id", "referer_version", name="uix_reference"),
    )
    if settings.DB_TYPE.lower() == "sqlite":
        primary_id: Mapped[int] = mapped_column(Integer, primary_key=True, autoincrement=True, name="id")
    else:
        primary_id: Mapped[int] = mapped_column(BigInteger, primary_key=True, autoincrement=True, name="id")

    space_id: Mapped[str] = mapped_column(String(100), nullable=False)
    referenced_type: Mapped[str] = mapped_column(String(20), nullable=False)
    referenced_id: Mapped[str] = mapped_column(String(100), nullable=False)
    referenced_version: Mapped[str] = mapped_column(String(100), nullable=False, default=DBFunBase.__version_none__)
    referer_type: Mapped[str] = mapped_column(String(20), nullable=False)
    referer_id: Mapped[str] = mapped_column(String(100), nullable=False)
    referer_version: Mapped[str] = mapped_column(String(100), nullable=False, default=DBFunBase.__version_none__)
    create_time: Mapped[int | None] = mapped_column(BigInteger, nullable=True)
    update_time: Mapped[int | None] = mapped_column(BigInteger, nullable=True)

    def __repr__(self) -> str:
        return f"<ReferenceDB(space_id='{self.space_id}', referenced={self.referenced_type}:{self.referenced_id}:{self.referenced_version}, referer={self.referer_type}:{self.referer_id}:{self.referer_version})>"


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (c) Huawei Technologies Co., Ltd. 2025-2025. All rights reserved
from typing import Dict, Type, TypeVar

from fastapi import HTTPException, status
from pydantic import BaseModel

from openjiuwen_studio.schemas.common import ResponseModel

T = TypeVar('T', bound=BaseModel)


def validate_request(request: Dict, model_class: Type[T]) -> T:
    """验证请求数据并转换为对应的模型类"""
    return model_class(**request)


def handle_response(res: ResponseModel) -> ResponseModel:
    if res.code == status.HTTP_200_OK:
        return res
    raise HTTPException(status_code=res.code, detail=res.message)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# Copyright (c) Huawei Technologies Co., Ltd. 2025-2025. All rights reserved.
import json
from typing import List, Any

from openjiuwen.core.common.logging import logger

from openjiuwen_studio.core.common import dsl
from openjiuwen_studio.core.common.dsl import ComponentType
from openjiuwen_studio.core.manager.convertor.components.end import end_outputs_convert
from openjiuwen_studio.core.manager.convertor.components.start import start_inputs_convert
from openjiuwen_studio.core.manager.internal.workflow import WorkflowCanvas
from openjiuwen_studio.schemas.node import NodeData
from openjiuwen_studio.core.manager.utils.utils import convert_to_properties_format
from openjiuwen_studio.schemas.workflow import WorkflowBase
from openjiuwen_studio.core.manager.convertor.component import component_convert
from openjiuwen_studio.core.manager.convertor.connection import connection_convert
from pydantic import ValidationError

from openjiuwen_studio.core.common.exceptions import JiuWenComponentException
from openjiuwen_studio.core.common.status_code import StatusCode
from openjiuwen_studio.core.manager.convertor.validators import validate_canvas_nodes


def extract_inputs_and_outputs_from_canvas(canvas_data: dict) -> tuple[list[dict[str, Any]], list[dict[str, Any]]]:
    """
    从原始canvas数据中提取inputs和outputs。

    Args:
        canvas_data: 原始的canvas字典数据

    Returns:
        tuple: (inputs, outputs)
    """
    try:
        nodes = canvas_data.get('nodes', [])

        inputs = []
        outputs = []

        for node in nodes:
            node_type = node.get('type')
            if node_type is None:
                continue

            try:
                node_type_int = int(node_type)

                if node_type_int == dsl.ComponentType.COMPONENT_TYPE_START:
                    data_dict = node.get('data', {})
                    # 将字典转换为NodeData对象
                    try:
                        node_data = NodeData(**data_dict)
                        inputs = start_inputs_convert(node_data)
                        logger.debug(f"Found start node, extracted inputs: {inputs}")
                    except ValidationError as e:
                        logger.warning(f"Failed to parse start node data: {e}, using empty inputs")
                        inputs = []

                elif node_type_int == dsl.ComponentType.COMPONENT_TYPE_END:
                    data_dict = node.get('data', {})
                    # 将字典转换为NodeData对象
                    try:
                        node_data = NodeData(**data_dict)
                        outputs = end_outputs_convert(node_data)
                        logger.debug(f"Found end node, extracted outputs: {outputs}")
                    except ValidationError as e:
                        logger.warning(f"Failed to parse end node data: {e}, using empty outputs")
                        outputs = []

            except (ValueError, TypeError) as e:
                logger.warning(f"Invalid node type '{node_type}', skipping: {e}")
                continue

        if not inputs:
            logger.warning("No start node found in workflow canvas data")
        if not outputs:
            logger.warning("No end node found in workflow canvas data")

        return inputs, outputs

    except Exception as e:
        logger.error(f"Failed to extract canvas data inputs/outputs: {e}")
        return [], []


def _friendly_validation_message(err: dict, schema: dict) -> tuple[str, str, int, str]:
    loc = err.get('loc', [])
    msg = err.get('msg', '')
    # 默认值
    component_id = ''
    component_type = 0
    field_path = '.'.join([str(p) for p in loc])
    # 提取node索引
    try:
        if 'nodes' in loc:
            idx = loc.index('nodes')
            node_idx = loc[idx + 1] if idx + 1 < len(loc) else None
            if isinstance(node_idx, int):
                node = schema.get('nodes', [])[node_idx]
                component_id = node.get('id', '')
                component_type = int(node.get('type', 0))
    except Exception as exc:
        logger.warning(
            "Failed to extract node info, use empty defaults | loc=%r node_idx=%r error=%s",
            loc, node_idx, exc
        )
        component_id = ''  # 与上面默认值保持一致
        component_type = 0

    def tuple_to_loc_str(tuple_path):
        return '_'.join(str(item) for item in tuple_path)

    err_loc = tuple_to_loc_str(loc)

    # 字段友好信息展示
    hint = msg + ", error location: " + err_loc
    is_missing_llm_model = (
            'llmParam' in field_path
            and field_path.endswith('model')
            and 'missing' in err.get('type', '')
    )
    if is_missing_llm_model:
        hint = '未配置模型，请在LLM组件中选择模型'
    elif msg.lower().startswith('field required'):
        hint = f"字段缺失: {field_path}"

    return hint, field_path, component_type, component_id


def workflow_convert(workflow_info: WorkflowBase, skip_validation: bool = False) -> dsl.Workflow:
    try:
        workflow_schema = json.loads(getattr(workflow_info, "workflow_schema", "{}") or "{}")
        try:
            canvas = WorkflowCanvas(**workflow_schema)
        except ValidationError as e:
            # 取第一条错误生成友好提示
            errs = e.errors() or []
            err = errs[0] if errs else {"msg": str(e), "loc": []}
            hint, field_path, component_type, component_id = _friendly_validation_message(err, workflow_schema)
            # 按组件类型选择错误码
            code = StatusCode.COMPONENT_CONFIG_INVALID.code
            if component_type == dsl.ComponentType.COMPONENT_TYPE_LLM:
                code = StatusCode.LLM_COMPONENT_CONFIG_INVALID.code
            raise JiuWenComponentException(
                code=code,
                message=StatusCode.COMPONENT_CONFIG_INVALID.errmsg.format(msg=hint),
                component_id=component_id,
                component_type=component_type,
                error_stage="validate"
            ) from e
        # 2. 业务逻辑校验（在 Pydantic 验证后、DSL 转换前）
        # 单组件调试时跳过全局校验，只验证被调试组件即可
        if not skip_validation:
            validate_canvas_nodes(canvas)

        # 3. DSL 转换
        nodes = canvas.nodes
        components = component_convert(canvas.edges, nodes, workflow_info.space_id, False)

        input_properties, input_requires = convert_to_properties_format(workflow_info.input_parameters)
        inputs = {
            "type": "object",
            "properties": input_properties,
            "required": input_requires
        }
        output_properties, output_requires = convert_to_properties_format(workflow_info.output_parameters)

        start_id: List[str] = []
        end_id: List[str] = []
        for component in components:
            if component.type == ComponentType.COMPONENT_TYPE_START:
                start_id.append(component.id)
            elif component.type == ComponentType.COMPONENT_TYPE_END:
                end_id.append(component.id)

        edges = canvas.edges
        connections = connection_convert(edges)

        version = getattr(workflow_info, "workflow_version", "1.0.0")

        workflow = dsl.Workflow(
            inputs=inputs,
            outputs=output_properties,
            start_id=start_id,
            end_id=end_id,
            id=getattr(workflow_info, "workflow_id", ""),
            name=getattr(workflow_info, "name", "Unnamed Workflow"),
            version=version,
            description=getattr(workflow_info, "desc", ""),
            components=components,
            connections=connections
        )

        return workflow
    except (json.JSONDecodeError, TypeError, AttributeError) as e:
        raise ValueError(f"Invalid workflow schema or input: {str(e)}") from e



#!/usr/bin/python3.10
# coding: utf-8
# Copyright (c) Huawei Technologies Co., Ltd. 2025-2025. All rights reserved
"""dependencies"""

__all__ = ["get_db_ops", "get_db_agent"]

from openjiuwen_studio.ops.modules.prompt.infra.database import get_db_agent, get_db_ops


from .user import UserCreate, UserUpdate, UserResponse, UserLogin
from .workflow import WorkflowSave, WorkflowResponseSave
from .workflow import WorkflowId, WorkflowBase, WorkflowCreate, WorkflowBaseResponse
from .model_config import ModelConfigCreate, ModelConfigUpdate, ModelConfigResponse
from .common import ResponseModel, PaginationParams
from .agent import AgentGetVersion

__all__ = [
    "UserCreate", "UserUpdate", "UserResponse", "UserLogin",
    "WorkflowId", "WorkflowBase", "WorkflowSave", "WorkflowCreate", "WorkflowBaseResponse",
    "ModelConfigCreate", "ModelConfigUpdate", "ModelConfigResponse",
    "ResponseModel", "PaginationParams",
    "AgentGetVersion"
]
